/*
 * ChromatoGate 1.2 for Windows
 *
 * Copyright (C) 2011 Nikolaos Ch. Alachiotis
 *
 * This program is free software; you may redistribute it and/or modify its
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. 
 *
 */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
 #include <windows.h>
 #include <tchar.h> 
 #include <strsafe.h>


 #define supported_functions 5

 #define CONSENSUS_RUN 1
 #define PRE_ALIGNMENT_RUN 2
 #define AMBIGUITIES_RUN 3 
 #define DIVERSITY_RUN 4 
 #define ONLINE_HELP 5
 
 int CONSENSUS_CALC_INSPECTION=-1;
 int DEFAULT_SCORING_SCHEME=-1;
 int CLOSEST_AMBIGUITY_REPLACEMENT=-1;
 int ONLINE_HELP_ARG=-1;

 int diversity_thres=-1;
 float diversity_thres_perc = -1.0;
 int total_number_of_files=-1;
 int is_percentage=0;
 int PSDsearch=-1;

 typedef struct
 {
	char * name; 
	int    F_or_R; 
	int	   st_pos_chrom; 
	int	   la_pos_chrom;
	int    length;
	char   sequence[1600];
	char   sequence2[1600];
 } seq_f;

 typedef struct
 {
	char * name; 
	int    F_or_R; 
	int	   st_pos_chrom; 
	int	   la_pos_chrom;
	int    length;
	char   sequence[1600];
	char   sequence2[1600];
 } consensus_entry;

 typedef struct
 {
	 int size;
	 char ** data;
 
 } cons_fw_list;

 cons_fw_list consensus_forward_list;
 int cons_fw_index;

 typedef struct
 { 
	 int size;
	 char ** data;
 
 } cons_rv_list;

 cons_rv_list consensus_reverse_list;
 int cons_rv_index;

 typedef struct
 {
	 int forward_index;
	 int reverse_index;
 } consensus_match_struct;

 char filepath[200];

 typedef struct
 {
 	int code;
 	int match;
	int match_vec[16];
 } div_r;

 typedef struct
 {
	 char seq_name[200];
	 int len_fw_uncut;
	 char * seq_fw_uncut;
	 int len_rv_uncut;
	 char * seq_rv_uncut;
 } seq_f_cons;

 seq_f_cons ** cons_list=NULL;

 seq_f * seq_d_table=NULL;
 char ** seq_n_table=NULL;

 seq_f * seq_d_table_ext=NULL;
 char ** seq_n_table_ext=NULL;
 char ** seq_filenames=NULL;

 char FORWARD_path[150];
 char REVERSE_path[150];
 char pre_alignment_fas_path[150];
 char pre_alignment_out_path[150];
 char lut_out_path[150];
 char consensus_out_path[150];
 char chromatogram_out_path[150];
 char Chromatograms_folder_path[150];
 char ma1_name[150];
 char ma2_name[150];
 int  ma2_length;
 char N_Report_path[150];
 char ma1_file_path[150];
 char D_Report_path[150];
 char A_Report_path[150];
 char Consensus_Forward_Path[150];
 char Consensus_Reverse_Path[150];
 char consensus_out_file[150];
 char consensus_fas_file[150];
 char consensus_inspection_folder_path[150];

 int run_arg=-1;
 int mode1_load_F,mode1_load_R,mode1_load_C;

 int ambtotal=0;
 int ambrtotal=0;
 
 char workspace_name_g[100];
 char ma1_filename[150];
 char ma2_filename[150];

 int U_case=-1;

 typedef struct
 {
	char * name; 
	int    F_or_R; 
	int	   st_pos_chrom; 
	int	   la_pos_chrom; 
	int    length;
	char   sequence[1600];
 } pre_seq_f;

 typedef struct
{
	char seq_name[100];
	int length;
	char * sequence;
	char * code; // 0 for MATCH, 1 for GAP in REVERSE, 2 for GAP in FORWARD, 3 for AMBIGUITY  

	int fw_cut_f;
	int fw_cut_l;
	int rv_cut_f;
	int rv_cut_l;

	int match_first_pos;
	int match_last_pos;

	int reverse_length;
	char * reverse_sequence;
	int forward_length;
	char * forward_sequence;

	int alignment_length;
	char * reverse_aligned_sequence;
	char * forward_aligned_sequence;

}consensus_struct;

 int consensus_out_size=-1;
 consensus_struct ** consensus_out;

 pre_seq_f * pre_seq_d_table=NULL;
 char ** pre_seq_n_table=NULL;

 int detect_fpos (int length, char * seq)
 {
	int position = 0, gap_s = 0;
	int i = 0 ;
	
	for(i=0;i<length;i++)
	{
		if (gap_s == 0)
		{
			if (seq[i]=='-')
			{
				gap_s = 1;
			}
		}
		else
		{
			if (seq[i]!='-')
			{
				gap_s = 0;
				position = i;
				break;
			}		
		}
	}
	return position + 1;
 }

 int detect_lpos (int st_pos, int length, char * seq)
 {
	int position = 0, gap_s = 0;
	int i = 0 ;
	
	for(i=st_pos-1;i<length;i++)
	{
		if (gap_s == 0)
		{
			if (seq[i]=='-')
			{
				position = i;
				break;
			}
		}
	}
	return position;
 }

 int detect_lpos_nos ( int length, char * seq)
 {
	int position = 0, gap_s = 0;
	int i = 0 ;
	
	for(i=0;i<length;i++)
	{
		if (gap_s == 0)
		{
			if (seq[i]=='-')
			{
				position = i;
				break;
			}
		}
	}
	return position;
 }

 int search_gaps(char * sequence, int length)
 {
	 int gaps=0,i=0;
	 for(i=0;i<length;i++)
	 {
		 if(sequence[i]=='-')
		 {
			 gaps=1;
			 break;
		 }
	 }
	 return gaps;
 }
 
 void cut_edges(pre_seq_f * temp)
 {
	int i=0,y=0;
	
	if(search_gaps(temp->sequence,temp->length)==1)
	{
		temp->st_pos_chrom = detect_fpos (temp->length, temp->sequence);
		temp->la_pos_chrom = detect_lpos (temp->st_pos_chrom, temp->length, temp->sequence);

		if(temp->la_pos_chrom==0)
		{	
			if(temp->st_pos_chrom<=temp->length/2)
			{
				temp->st_pos_chrom;
				temp->la_pos_chrom = temp->length;
			}
			else
			{
				temp->st_pos_chrom = 1 ;
				temp->la_pos_chrom = detect_lpos_nos (temp->length, temp->sequence);
			}			
		}
	}
	else
	{
		temp->st_pos_chrom = 1 ;
		temp->la_pos_chrom = temp->length;
	}
	
	for(i=temp->st_pos_chrom-1;i<temp->la_pos_chrom;i++)
	{
		temp->sequence[y++]=temp->sequence[i];
	}
	temp->sequence[y]=0;
 }

 void rev_comp(pre_seq_f * temp)
 {
	int i=0,y=0;
	char temp_character;
	char tmp_seq[1600];
	
	for(i=temp->la_pos_chrom - temp-> st_pos_chrom ; i>-1 ; i--)
	{	
		temp_character = temp -> sequence[i];
		
		if(temp_character=='A'){tmp_seq[y++]='T';}
		if(temp_character=='C'){tmp_seq[y++]='G';}
		if(temp_character=='G'){tmp_seq[y++]='C';}
		if(temp_character=='T'){tmp_seq[y++]='A';}
		if(temp_character=='N'){tmp_seq[y++]='N';}	
		if(temp_character=='U'){tmp_seq[y++]='A';}
		if(temp_character=='M'){tmp_seq[y++]='M';}
		if(temp_character=='R'){tmp_seq[y++]='R';}
		if(temp_character=='W'){tmp_seq[y++]='W';}
		if(temp_character=='S'){tmp_seq[y++]='S';}
		if(temp_character=='Y'){tmp_seq[y++]='Y';}
		if(temp_character=='V'){tmp_seq[y++]='V';}
		if(temp_character=='H'){tmp_seq[y++]='H';}
		if(temp_character=='D'){tmp_seq[y++]='D';}
		if(temp_character=='B'){tmp_seq[y++]='B';}
	}
	
	for(i=0;i<temp->la_pos_chrom - temp-> st_pos_chrom + 1;i++)
	{
		temp -> sequence[i] = tmp_seq[i];
	}
	
 }

 pre_seq_f get_seq_F_special(FILE *fp,int index,char * filename)
 {
	char t_ch;
	char done;
	int c_ind=0;
	int i=0;
	int charind=0;
	pre_seq_f temp;

	fscanf(fp,"%s",pre_seq_n_table[index]);


	t_ch = fgetc(fp);

	t_ch = fgetc(fp);

	charind++;
	while(t_ch!=EOF)
	{
		if (t_ch=='A' || t_ch=='C' || t_ch=='G' || t_ch=='T' || t_ch=='-' || t_ch=='N' ||
			t_ch=='U' || t_ch=='M' || t_ch=='R' || t_ch=='W' || t_ch=='S' || t_ch=='Y' ||
			t_ch=='K' || t_ch=='V' || t_ch=='H' || t_ch=='D' || t_ch=='B' ) 
		{
			temp.sequence[c_ind++]=t_ch;
		}
		else
		{
			if(t_ch!='\n' && t_ch!=EOF && (t_ch>=32 && t_ch<=126))
			{
				
				printf("\n\n\n\n   => Invalid character (%c-%d) in file: %s.\n\n",t_ch,charind,filename);				
        
				printf("\n  Press Enter to exit...\n\n");
				scanf("%c",&done);
		
				exit(0);
			}
	    }
		t_ch = fgetc(fp);

		charind++; // TODO
	}
	
	temp.length = c_ind;

	temp.st_pos_chrom = 1;
	temp.la_pos_chrom = temp.length;

	temp.F_or_R = 2;	

	return temp;
 }

 pre_seq_f get_seq_F(FILE *fp,int index,char * filename)
 {
	char t_ch;
	char done;
	int c_ind=0;
	int i=0;
	pre_seq_f temp;

	fscanf(fp,"%s",pre_seq_n_table[index]);
	
	t_ch = fgetc(fp);
	

	t_ch = fgetc(fp);
	while(t_ch!=EOF)
	{
		if (t_ch=='A' || t_ch=='C' || t_ch=='G' || t_ch=='T' || t_ch=='-' || t_ch=='N' ||
			t_ch=='U' || t_ch=='M' || t_ch=='R' || t_ch=='W' || t_ch=='S' || t_ch=='Y' ||
			t_ch=='K' || t_ch=='V' || t_ch=='H' || t_ch=='D' || t_ch=='B' ) 
		{
			temp.sequence[c_ind++]=t_ch;
		}
		else
		{
			if((t_ch!='\n') && (t_ch>=32 && t_ch<=126))
			{
				
				printf("\n\n\n\n   => Invalid character (%c) in file: %s.\n\n",t_ch,filename);				

				printf("\n  Press Enter to exit...\n\n");
				scanf("%c",&done);

				exit(0);
			}

		}
		t_ch = fgetc(fp);
	}
	
	temp.length = c_ind;

	cut_edges (&temp);
	
	temp.F_or_R = 0;	

	return temp;
 }

 pre_seq_f get_seq_R(FILE *fp, int index, char * filename)
 {
	char t_ch;
	char done;
	int c_ind=0;
	int i=0;
	pre_seq_f temp;

	fscanf(fp,"%s",pre_seq_n_table[index]);
	
	t_ch = fgetc(fp);
	
	t_ch = fgetc(fp);
	while(t_ch!=EOF)
	{
		if (t_ch=='A' || t_ch=='C' || t_ch=='G' || t_ch=='T' || t_ch=='-' || t_ch=='N' ||
			t_ch=='U' || t_ch=='M' || t_ch=='R' || t_ch=='W' || t_ch=='S' || t_ch=='Y' ||
			t_ch=='K' || t_ch=='V' || t_ch=='H' || t_ch=='D' || t_ch=='B' ) 
		{
			temp.sequence[c_ind++]=t_ch;
		}
		else
		{
			if((t_ch!='\n') && (t_ch>=32 && t_ch<=126))
			{
				
				printf("\n\n\n\n   => Invalid character (%c) in file: %s.\n\n",t_ch,filename);				

				printf("\n  Press Enter to exit...\n\n");
				scanf("%c",&done);

				exit(0); 
			}
		}
		t_ch = fgetc(fp);
	}
	
	temp.length = c_ind;

	cut_edges (&temp);
	
	rev_comp (&temp);
	
	temp.F_or_R = 1;	

	return temp;
 }

int read_all_X_files(int mode,int prev_table_index, int F_or_R, char * workspace, char * folder)
{
	int i=0;
	int y=0;
	int z=0;
	int skip=0;
	WIN32_FIND_DATA ffd;
	TCHAR szDir[100];
	TCHAR tmp[100];
	FILE *fpF;
	FILE *fpR;
	int table_index=0;
	char temp_filename[200];
	char temp_filename2[400];
	int max_F_num;
	int checkmode=-1;

	HANDLE hFind = INVALID_HANDLE_VALUE;

	table_index=prev_table_index;

	for(i=0;i<strlen(workspace);i++)
	{
		tmp[i]=workspace[i];
	}
	tmp[i]=0;

	SetCurrentDirectory(tmp);

	for(i=0;i<strlen(folder);i++)
	{
		szDir[i]=folder[i];
	}
	szDir[i]=0;

	StringCchCat(szDir, 100, TEXT("\\*"));
	
	hFind = FindFirstFile(szDir, &ffd);

	do
    {
      if (ffd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
      {

      }
      else
      {


		  i=0;
		  while(ffd.cFileName[i]!=0)
		  {
			  temp_filename[i] = ffd.cFileName[i];
			  i++;		
		  }
		  temp_filename[i]=0;

		  if(temp_filename[strlen(temp_filename)-4]=='.')
		  {
			if(temp_filename[strlen(temp_filename)-3]=='f' && temp_filename[strlen(temp_filename)-2]=='a' && temp_filename[strlen(temp_filename)-1]=='s')
			{
		  

				  i=0;
				  if(F_or_R==0)
				  {
					temp_filename2[i++]='F';temp_filename2[i++]='O';temp_filename2[i++]='R';temp_filename2[i++]='W';
					temp_filename2[i++]='A';temp_filename2[i++]='R';temp_filename2[i++]='D';temp_filename2[i++]='/';
				  }
				  else
				  {
					temp_filename2[i++]='R';temp_filename2[i++]='E';temp_filename2[i++]='V';temp_filename2[i++]='E';
					temp_filename2[i++]='R';temp_filename2[i++]='S';temp_filename2[i++]='E';temp_filename2[i++]='/';
				  }
				  for(y=i;y<strlen(temp_filename)+i;y++)
				  {
					  temp_filename2[y]=temp_filename[z++];
				  }	
				  z=0;
				  temp_filename2[y]=0;

				  if(F_or_R==0)
				  {
					fpF = fopen(temp_filename2,"r");

					if(mode==0)
					{
					  mode1_load_F++;
					}

				  }
				  else
				  {
					fpR = fopen(temp_filename2,"r");

					if(mode==0)
					{
					   mode1_load_R++;
					}
				  }

				  if(F_or_R==0)
				  {
					  if(mode==1)
					  {
						pre_seq_d_table[table_index++]=get_seq_F(fpF,table_index,temp_filename2);
					  }
				  }
				  else
				  {
					  if(mode==1)
					  {
						pre_seq_d_table[table_index++]=get_seq_R(fpR,table_index,temp_filename2);
					  }
				  }
			}
		  }
      }
   }
   while (FindNextFile(hFind, &ffd) != 0);

   tmp[0]='.';
   tmp[1]='.';
   tmp[2]=0;
   SetCurrentDirectory(tmp);

   return table_index;
}

int read_all_X_files_consensus(int mode,int prev_table_index, int F_or_R, char * workspace, char * folder)
{
	int i=0;
	int i2=0;
	int y=0;
	int z=0;
	int skip=0;
	char done;
	int consensus_forward_load=0, consensus_reverse_load=0;
	WIN32_FIND_DATA ffd;
	TCHAR szDir[100];
	TCHAR tmp[100];
	FILE *fpF;
	FILE *fpR;
	int table_index=0;
	char temp_filename[200];
	char temp_filename2[400];
	int max_F_num;
	int checkmode=-1;

	HANDLE hFind = INVALID_HANDLE_VALUE;

	table_index=prev_table_index;

	for(i=0;i<strlen(workspace);i++)
	{
		tmp[i]=workspace[i];
	}
	tmp[i]=0;

	SetCurrentDirectory(tmp);

	i=0;
	szDir[i++]='C';	szDir[i++]='O';	szDir[i++]='N';	szDir[i++]='S';	szDir[i++]='E';
	szDir[i++]='N';	szDir[i++]='S';	szDir[i++]='U'; szDir[i++]='S';	szDir[i++]='/';
	
	y=i;
	for(i=y;i<y+strlen(folder);i++)
	{
		szDir[i]=folder[i-y];
	}
	szDir[i]=0;
	
	StringCchCat(szDir, 100, TEXT("\\*"));
	
	hFind = FindFirstFile(szDir, &ffd);

   do
   {  

      if (ffd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
      {

      }
      else
      {


		 
		  i=0;
		  while(ffd.cFileName[i]!=0)
		  {
			  temp_filename[i] = ffd.cFileName[i];
			  i++;		
		  }
		  temp_filename[i]=0;

		  if(temp_filename[strlen(temp_filename)-4]=='.')
		  {
			if(temp_filename[strlen(temp_filename)-3]=='f' && temp_filename[strlen(temp_filename)-2]=='a' && temp_filename[strlen(temp_filename)-1]=='s')
			{

				 i=0;
				  temp_filename2[i++]='C';temp_filename2[i++]='O';temp_filename2[i++]='N';
				  temp_filename2[i++]='S';temp_filename2[i++]='E';temp_filename2[i++]='N';
				  temp_filename2[i++]='S';temp_filename2[i++]='U';temp_filename2[i++]='S';temp_filename2[i++]='/';

				  if(F_or_R==0)
				  {

					temp_filename2[i++]='F';temp_filename2[i++]='O';temp_filename2[i++]='R';temp_filename2[i++]='W';
					temp_filename2[i++]='A';temp_filename2[i++]='R';temp_filename2[i++]='D';temp_filename2[i++]='/';
				  }
				  else
				  {
					temp_filename2[i++]='R';temp_filename2[i++]='E';temp_filename2[i++]='V';temp_filename2[i++]='E';
					temp_filename2[i++]='R';temp_filename2[i++]='S';temp_filename2[i++]='E';temp_filename2[i++]='/';
				  }

				  i2=0;
				  for(y=i;y<strlen(temp_filename)+i;y++)
				  {
					  temp_filename2[y]=temp_filename[z++];
		  
					  if(mode==1)
					  {
						  if(F_or_R==0)
						  {
							  consensus_forward_list.data[cons_fw_index][i2]=temp_filename2[y];
							  consensus_forward_list.data[cons_fw_index][i2+1]=0;
							  i2++;						  
						  }
						  else
						  {
							  consensus_reverse_list.data[cons_rv_index][i2]=temp_filename2[y];
							  consensus_reverse_list.data[cons_rv_index][i2+1]=0;
							  i2++;
						  }
					  }
				  }

				  if(mode==1)
				  {
					 if(F_or_R==0)
					 {
						 cons_fw_index++;
					 }
					 else
					 {
						 cons_rv_index++;
					 }

				  }
				  z=0;
				  temp_filename2[y]=0;

		  
				  if(F_or_R==0)
				  {
					fpF = fopen(temp_filename2,"r");

					if(mode==0)
					{
					  consensus_forward_load++;
					}
				  }
				  else
				  {
					fpR = fopen(temp_filename2,"r");

					if(mode==0)
					{
					   consensus_reverse_load++;
					}
				  }
				
			}
			else
			{
				//printf("\n\n\n\n   => File \"%s\" is not a .FAS file! \n\n      Remove it from the folder \"../CONSENSUS/%s\" and try again!\n\n",temp_filename,folder);				

				//printf("\n  Press Enter to exit...\n\n");
				//scanf("%c",&done);

				//exit(0);
			}

		  }	 	
      }
   }
   while (FindNextFile(hFind, &ffd) != 0);

   tmp[0]='.';
   tmp[1]='.';
   tmp[2]=0;
   SetCurrentDirectory(tmp);

   if(F_or_R==0)
   {
	  return consensus_forward_load;   
   }
   else
   {
	  return consensus_reverse_load;
   }

   return 0;
   
}

int read_all_consensus_files_for_pre_alignment(int mode,int index, char * workspace)
{
	int i=0;
	int i2=0;
	int y=0;
	int z=0;
	int skip=0;
	char done;
	int consensus_forward_load=0, consensus_reverse_load=0;
	WIN32_FIND_DATA ffd;
	TCHAR szDir[100];
	TCHAR tmp[100];
	FILE *fpF;
	FILE *fpR;
	FILE *fpC;
	int table_index=0;
	char temp_filename[200];
	char temp_filename2[400];
	int max_F_num;
	int checkmode=-1;
	int load=0;
	int index_local=index;
	char tmp_file_char;
	int fas_file_y;

	HANDLE hFind = INVALID_HANDLE_VALUE;

	for(i=0;i<strlen(workspace);i++)
	{
		tmp[i]=workspace[i];
	}
	tmp[i]=0;

	SetCurrentDirectory(tmp);

	i=0;
	szDir[i++]='C';	szDir[i++]='O';	szDir[i++]='N';	szDir[i++]='S';	szDir[i++]='E';
	szDir[i++]='N';	szDir[i++]='S';	szDir[i++]='U'; szDir[i++]='S';	szDir[i++]=0;
	
    StringCchCat(szDir, 100, TEXT("\\*"));
	
    hFind = FindFirstFile(szDir, &ffd);

	do
	{  
		fas_file_y=0;

		if (ffd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
		{

		}
		else
		{
		  i=0;
		  while(ffd.cFileName[i]!=0)
		  {
			  temp_filename[i] = ffd.cFileName[i];
			  i++;		
		  }
		  temp_filename[i]=0;

		  if(temp_filename[strlen(temp_filename)-4]=='.')
		  {
			if(temp_filename[strlen(temp_filename)-3]=='f' && temp_filename[strlen(temp_filename)-2]=='a' && temp_filename[strlen(temp_filename)-1]=='s')
			{
				load++;
		
				if(mode==0)
				{
					mode1_load_C = load;
				}
				else
				{				
					i=0;
					temp_filename2[i++]='C';temp_filename2[i++]='O';temp_filename2[i++]='N';
					temp_filename2[i++]='S';temp_filename2[i++]='E';temp_filename2[i++]='N';
					temp_filename2[i++]='S';temp_filename2[i++]='U';temp_filename2[i++]='S';
					temp_filename2[i++]='\\';

					for(i2=i;i2<strlen(temp_filename)+i;i2++)
					{
						temp_filename2[i2]=temp_filename[i2-i];
					}
					temp_filename2[i2]=0;



					fpC = fopen(temp_filename2,"r");


					if(fpC==NULL)
					{
					
					}
					else
					{					
						pre_seq_d_table[index_local++]=get_seq_F_special(fpC,index_local,temp_filename2);
					
						fclose(fpC);
					}
				}
			}
			else
			{
				
			}
		  }
		}
	}
	while (FindNextFile(hFind, &ffd) != 0);

	tmp[0]='.';
	tmp[1]='.';
	tmp[2]=0;
	SetCurrentDirectory(tmp);
		
	return 0;
   
}

 void read_data()
 {

	int type = 0;
	int i=0;
	FILE * fpF ;
	FILE * fpR ;
	FILE *fpC_check;
	FILE *fpC;
	TCHAR tempdirect[200];
	char workspacedirectory[200];
	char foldern[8];
	char foldern2[12];
	char done;	
	int table_index=0;

	for(i=0;i<200;i++)
	{
		workspacedirectory[i]=FORWARD_path[i];

		if(workspacedirectory[i]=='/')
		{
			workspacedirectory[i]=0;
			break;
		}
	}


	i=0;
	foldern[i++]='F';foldern[i++]='O';foldern[i++]='R';foldern[i++]='W';
	foldern[i++]='A';foldern[i++]='R';foldern[i++]='D';foldern[i++]=0;

	table_index=read_all_X_files(0,0,0, workspacedirectory, foldern);
	
	i=0;
	foldern[i++]='R';foldern[i++]='E';foldern[i++]='V';foldern[i++]='E';
	foldern[i++]='R';foldern[i++]='S';foldern[i++]='E';foldern[i++]=0;
	
	read_all_X_files(0,table_index,1, workspacedirectory, foldern);

	fpC_check = fopen(consensus_out_path,"r");

	if(fpC_check==NULL)
	{
		mode1_load_C=0;
	}
	else
	{
		mode1_load_C=0;
		read_all_consensus_files_for_pre_alignment(0,0,workspacedirectory);
	}
	
	total_number_of_files=mode1_load_F + mode1_load_R + mode1_load_C;		

	pre_seq_d_table = (pre_seq_f*)malloc(sizeof(pre_seq_f)*total_number_of_files);

	pre_seq_n_table = (char**)malloc(sizeof(char *)*total_number_of_files);

	for (i=0;i<total_number_of_files;i++)
	{
		pre_seq_n_table[i]=(char*)malloc(sizeof(char)*50);
	}

	i=0;
	foldern[i++]='F';foldern[i++]='O';foldern[i++]='R';foldern[i++]='W';
	foldern[i++]='A';foldern[i++]='R';foldern[i++]='D';foldern[i++]=0;

	table_index=read_all_X_files(1,0,0, workspacedirectory, foldern);
	
	i=0;
	foldern[i++]='R';foldern[i++]='E';foldern[i++]='V';foldern[i++]='E';
	foldern[i++]='R';foldern[i++]='S';foldern[i++]='E';foldern[i++]=0;
	
	read_all_X_files(1,table_index,1, workspacedirectory, foldern);
 
	read_all_consensus_files_for_pre_alignment(1,mode1_load_F + mode1_load_R,workspacedirectory);
	
 }

 void print_pre_seq_f_data_file(FILE *fp,pre_seq_f * temp)
 {
	int i=0;	

	for(i=0;i<temp->la_pos_chrom - temp-> st_pos_chrom + 1;i++)
	{
		fprintf(fp,"%c",temp-> sequence[i]);
	}

 }

 void create_pre_multalign_file(int size, FILE *fp)
 {
	int i=0;
	
	for(i=0;i<size;i++)
	{
		fprintf(fp, "%s\n",pre_seq_n_table[i]);
		print_pre_seq_f_data_file(fp,&pre_seq_d_table[i]);
		fprintf(fp,"\n");
	}	
 }

 void store_pre_multalign_data(int size, FILE *fp)
 { 
	int i=0;
	for(i=0;i<size;i++)
	{
		fprintf(fp,"%d %d %d %d %d\n",i+1,pre_seq_d_table[i].F_or_R,pre_seq_d_table[i].st_pos_chrom,pre_seq_d_table[i].la_pos_chrom,pre_seq_d_table[i].length);
		print_pre_seq_f_data_file(fp,&pre_seq_d_table[i]);
		fprintf(fp,"\n");
	}	
 }

 void print_lut_out(int size, FILE *fp)
 {
 	int i=0;
 	fprintf(fp, "%d\n",size);
 	for(i=0;i<size;i++)
 	{
 		fprintf(fp, "%d\t%s\n",i+1,pre_seq_n_table[i]);
 	}	
 }

   
 void read_lut(int size, FILE *fp)
 {	
 	int tmp = 0,i;
 	
 	for(i=0;i<size;i++)
 	{
 		fscanf(fp,"%d",&tmp);
 		fgetc(fp);
 		fgetc(fp);
 		fscanf(fp,"%s",seq_n_table[i]);
 	}
 }

 void read_pre_alignment(int size, FILE *fp)
 {	
 	int tmp = 0,i;
 	char tmpseq[1600];
 	
 	for(i=0;i<size;i++)
 	{
 		fscanf(fp,"%d",&tmp);
 		fscanf(fp,"%d",&seq_d_table[i].F_or_R);
 		fscanf(fp,"%d",&seq_d_table[i].st_pos_chrom);
 		fscanf(fp,"%d",&seq_d_table[i].la_pos_chrom);
 		fscanf(fp,"%d",&seq_d_table[i].length);
 		fscanf(fp,"%s",tmpseq);
  	}
 } 
 
 int get_FASTA_seq_length (FILE *fp)
 {
 	int length=0;
 	char tmpchar;
 	char tmpseq[80];
 	int stop = 0;
 	tmpchar = fgetc(fp);
 	while(tmpchar!='\n')
 	{
 		tmpchar = fgetc(fp);
 	}
 	
 	while(stop==0)
 	{
		tmpchar = fgetc(fp);
	
		if(tmpchar=='>') {stop=1;}else{stop=0;}
	
		if(tmpchar=='A' || tmpchar=='C' || tmpchar=='G' || tmpchar=='T' || tmpchar=='N' || tmpchar=='-' ||
		   tmpchar=='K' || tmpchar=='U' || tmpchar=='M' || tmpchar=='R' || tmpchar=='W' || tmpchar=='S' ||
		   tmpchar=='Y' || tmpchar=='V' || tmpchar=='H' || tmpchar=='D' || tmpchar=='B' )
		{
			length++;
		}
 	} 	
 	
 	return length;
 }

 char * get_seq_name(char * in)
 {
	 char * out = (char *)malloc(sizeof(char) * 100);
	 int i;
	 int len = strlen(in);

	 for(i=0;i<len;i++)
	 {
		 out[i]=in[i+1];
		 out[i+1]=0;
	 }

	 return out;

 }


 int read_multiple_alignment(int size, int length, FILE *fp, int seq)
 {	
 	char tmpchar;
 	char tmpchars[100], * seq_name;
 	int i=0, seq_len=0;
	int set_U_case=1;
	int matpos=-1,sel_ext=0,ext_i=0;
	int num_of_seqs=0, extra_seqs=0;
	char done;
	int size_all=0;

	tmpchar=fgetc(fp);

	while(tmpchar!=EOF)
	{
		if(tmpchar=='>')
		{
			num_of_seqs++;
		}

		tmpchar=fgetc(fp);
	}

	if(num_of_seqs<size)
	{
		printf("\n\n\n\n   => %d missing sequence(s) from the alignment file!\n\n",size-num_of_seqs);
		printf("      Update the alignment file and try again.\n\n");

		printf("\n  Press Enter to exit...\n\n");
		scanf("%c",&done);scanf("%c",&done);

		exit(0); 

	}
	else
	{
		size_all = size;

		if(num_of_seqs>size)
		{
			size_all =  num_of_seqs;

			extra_seqs = num_of_seqs-size;				

			seq_d_table_ext = (seq_f*)malloc(sizeof(seq_f)*extra_seqs);
	 	
	 		seq_n_table_ext = (char**)malloc(sizeof(char *)*extra_seqs);
			for (i=0;i<extra_seqs;i++)
			{
				seq_n_table_ext[i]=(char*)malloc(sizeof(char)*50);
			}
		}
	}


	rewind(fp);
 	ext_i=-1;
	
 	for(i=0;i<size_all;i++)
 	{
 		fscanf(fp,"%s",tmpchars);

		seq_name = get_seq_name(tmpchars);
		matpos = match_seq_name(size, seq_name) - 1;

		sel_ext=0;
		if(matpos==-2)
		{			
			sel_ext=1;
			ext_i++;
			seq_n_table_ext[ext_i] = seq_name;

		}
 
 		while(seq_len!=length)
 		{
 			tmpchar = fgetc(fp);
 			if(tmpchar=='A' || tmpchar=='C' || tmpchar=='G' || tmpchar=='T' || tmpchar=='N' || tmpchar=='-' ||
			   tmpchar=='K' || tmpchar=='U' || tmpchar=='M' || tmpchar=='R' || tmpchar=='W' || tmpchar=='S' ||
			   tmpchar=='Y' || tmpchar=='V' || tmpchar=='H' || tmpchar=='D' || tmpchar=='B' )
			{	
				if (tmpchar=='U')
				{
					if(set_U_case==1)
					{
						set_U_case=0;
						U_case=1;
					}
				}

				
				if(seq==1)
				{	
					if(sel_ext==0)
					{
						seq_d_table[matpos].sequence[seq_len++]=tmpchar;
						seq_d_table[matpos].sequence[seq_len+1]=0;
					}
					else
					{
						seq_d_table_ext[ext_i].sequence[seq_len++]=tmpchar;
						seq_d_table_ext[ext_i].sequence[seq_len+1]=0;
					}
				}
				else
				{
					if(sel_ext==0)
					{
						seq_d_table[matpos].sequence2[seq_len++]=tmpchar;
						seq_d_table[matpos].sequence2[seq_len+1]=0;
					}
					else
					{
						seq_d_table_ext[ext_i].sequence2[seq_len++]=tmpchar;
						seq_d_table_ext[ext_i].sequence2[seq_len+1]=0;
					}
				}				
			}
 		}
 		seq_len = 0; 
 	} 

	return size_all;
 } 
 
 int remove_gaps (int position, char * sequence)
 {
 	int value=0,i,Ls=0;
 	
 	for(i=0;i<position;i++)
 	{
 		if(sequence[i]=='-')
 		{
 			value++;
 		}
		if(sequence[i]=='L')
		{
			printf("\nIn here");
			Ls++;
		}

 	}
 	
 	return value-Ls;
 }

  int count_Lgaps (int position, char * sequence)
 {
 	int value=0,i;
 	
 	for(i=0;i<position;i++)
 	{
 		if(sequence[i]=='L')
 		{
 			value++;
 		}
 	}
 	
 	return value;
 }

 

 int get_cons_out_index(int y_ind)
 {
	 int i;
	 int index;
	 char done;

	 index = -1;

	 for(i=0;i<consensus_out_size;i++)
	 {
 
		 if(strcmp(consensus_out[i]->seq_name,seq_n_table[y_ind])==0)
 		 {
			index=i;
 			break;
 		 }
	 }

	 if(index==-1)
	 {
		 printf("\n\n\n\n   => Index error occured during consensus processing.\n\n");
		 printf("      Please repeat from step 1 to resolve this issue!\n\n");
		 
		 printf("\n  Press Enter to exit...\n\n");
		 scanf("%c",&done);scanf("%c",&done);

		 exit(0);
	 }

	 return index;
 }

 int count_codes(char code,int cons_out_index,int x_ind_pos)
 {
	 int total=0;
	 int i;

	 for(i=0;i<x_ind_pos-1;i++)
	 {
		 if(consensus_out[cons_out_index]->code[i]=='0' || consensus_out[cons_out_index]->code[i]==code || consensus_out[cons_out_index]->code[i]=='3' )
		 {
			 total++;
		 }
	 }

	 return total;
 }
		 

 int get_cons_chrom_pos(int y_ind, int x_ind, int fw_or_rv)
 {
	 int position=-1;
	 int cons_out_index = -1;
	 int x_ind_pos;
	 int fw_nucls,rv_nucls;
	 int position_fw, position_rv;

	 cons_out_index = get_cons_out_index(y_ind);
	 
	 x_ind_pos = x_ind + 1;

	 if(x_ind_pos < consensus_out[cons_out_index]-> match_first_pos)
	 {
		 position_fw = 0;
		 position_rv = consensus_out[cons_out_index]->rv_cut_l - x_ind_pos + 1;
		 
	 }
	 else
	 {
		 if(consensus_out[cons_out_index]-> match_first_pos <= x_ind_pos &&  x_ind_pos <= consensus_out[cons_out_index]->match_last_pos)
		 {
			 rv_nucls=count_codes('2',cons_out_index,x_ind_pos);
			 position_rv = consensus_out[cons_out_index]->rv_cut_l - rv_nucls;

			 fw_nucls = count_codes ('1',cons_out_index,x_ind_pos);
			 position_fw = consensus_out[cons_out_index]->fw_cut_f + fw_nucls;

		 }
		 else
		 {
			 fw_nucls = count_codes ('1',cons_out_index,x_ind_pos);
			 position_fw = consensus_out[cons_out_index]->fw_cut_f + fw_nucls;
			 position_rv = 0;
		 }		 
	 }

	 if (fw_or_rv == 0)
	 {
		 position = position_fw;
	 }
	 else
	 {
		 position = position_rv;
	 }

	 return position;
 }

 int get_cons_chrom_pos_n(int y_ind, int x_ind, int fw_or_rv, int nogaps_ind)
 {
	 int position=-1;
	 int cons_out_index = -1;
	 int x_ind_pos;
	 int fw_nucls,rv_nucls;
	 int position_fw, position_rv;
	 char selection_case='0';
	 
	 cons_out_index = get_cons_out_index(y_ind);
	 
	 x_ind_pos = nogaps_ind + 1;

	 selection_case = consensus_out[cons_out_index]->code[nogaps_ind];

	 rv_nucls = count_codes('2',cons_out_index,x_ind_pos);
	 fw_nucls = count_codes('1',cons_out_index,x_ind_pos);

	 if(selection_case == '2')
	 {
		 position_rv = consensus_out[cons_out_index]->rv_cut_l - rv_nucls;
		 position_fw = 0;
	 }
	 else
	 {
		 if(selection_case == '0' || selection_case=='3')
		 {
			 position_rv = consensus_out[cons_out_index]->rv_cut_l - rv_nucls;
			 position_fw = consensus_out[cons_out_index]->fw_cut_f + fw_nucls;
		 }
		 else // selection case == '1'
		 {
			 position_rv = 0;
			 position_fw = consensus_out[cons_out_index]->fw_cut_f + fw_nucls;
		 }

	 }

	 if (fw_or_rv == 0)
	 {
		 position = position_fw;
	 }
	 else
	 {
		 position = position_rv;
	 }

	 return position;
 }
 
 void store_N_position(int index, FILE *fpN, int y_ind, int x_ind)
 {
 	int gaps=0,chrom_N_pos,chrom_N_pos_FW,chrom_N_pos_RV, cons_code_pos;

	if (fpN==NULL){return;}
 	
 	gaps=remove_gaps(x_ind,seq_d_table[y_ind].sequence);
 	
 	if (seq_d_table[y_ind].F_or_R == 0)
 	{
 		chrom_N_pos = x_ind - gaps + seq_d_table[y_ind].st_pos_chrom ;
 		
		fprintf(fpN, "\n\n   %d\t\t%s\n\t\t\t\t\t\t\t\t%d\t\t\t\tFW.%d\t\t\t\t>?\n\t_____________________________________________________________________________________________________________________________\n", index, seq_n_table[y_ind],x_ind+1,chrom_N_pos);
 	
	}
 	else
 	{
		if (seq_d_table[y_ind].F_or_R == 1)
		{
 			chrom_N_pos = seq_d_table[y_ind].la_pos_chrom - x_ind + gaps ;
	
			fprintf(fpN, "\n\n   %d\t\t%s\n\t\t\t\t\t\t\t\t%d\t\t\t\tRV.%d\t\t\t\t>?\n\t_____________________________________________________________________________________________________________________________\n", index, seq_n_table[y_ind],x_ind+1,chrom_N_pos);
 	
		}
		else // case: consensus
		{
			cons_code_pos = x_ind - gaps;

			chrom_N_pos_FW = get_cons_chrom_pos_n(y_ind,x_ind,0,cons_code_pos);
			chrom_N_pos_RV = get_cons_chrom_pos_n(y_ind,x_ind,1,cons_code_pos);

			if(chrom_N_pos_FW==0 && chrom_N_pos_RV==0)
			{
				exit(0);
			}

			fprintf(fpN, "\n\n   %d\t\t%s\n\t\t\t\t\t\t\t\t%d\t\t\t\tFW.%d - RV.%d\t\t\t>?\n\t_____________________________________________________________________________________________________________________________\n", index, seq_n_table[y_ind],x_ind+1,chrom_N_pos_FW,chrom_N_pos_RV);
 	
		}
 	} 																			
 }
 
 div_r diversity_check (int * countvec, int countA, int countC, int countG, int countT, int countN, int countM, int countR, int countW, int countS, int countY, int countK, int countV, int countH, int countD, int countB, int countGAP)
 {
	 int i;
	 int total_no_gaps=0;
	 int thres1, thres2;
	 int match;
	 int match_y;
	 int events;
	 div_r result;

	 thres1 = diversity_thres;

	 if(PSDsearch==2) // Total Sum Strategy
	 {

		 for(i=0;i<15;i++)
		 {
			total_no_gaps = total_no_gaps + countvec[i]; 
		 }
	
		 thres2 = total_no_gaps - thres1;

		 for(i=0;i<15;i++)
		 {
			if(countvec[i]>=thres2 && countvec[i]!=total_no_gaps)
			{
				match=1;
				break;
			}
			else
			{
				match=0;
			}
		 }

		 if(match==1)
		 {
			events=0;
			match_y=0;
			for(i=0;i<15;i++)
			{
				if(countvec[i]<=thres1 && countvec[i]!=0)
				{
					result.match_vec[i]=1;
					events++;
					if(match_y==0)
					{
						match_y++;
					}
				}
				else
				{
					result.match_vec[i]=0;
				}
			  
			}
			if(match_y==1)
			{ 
				if(events<=thres1)
				{
					result.code=1;
					result.match=1;
					return result;
				}
			}
		
		 }

		 result.code=-1;
		 result.match=-1;
	 
		 for(i=0;i<15;i++)
		 {
			 result.match_vec[i]=0; 
		 }
		 return result;
	 }
	 else
	 {
		 match_y=0;
		 for(i=0;i<15;i++)
		 {
			if(countvec[i]<=thres1 && countvec[i]!=0)
			{
				result.match_vec[i]=1;
				if(match_y==0)
				{
					match_y++;
				}
			}
			else
			{
				result.match_vec[i]=0;
			}
		}
        if(match_y==1)
		{ 
			result.code=1;
			result.match=1;
			return result;
			
		}
		
		result.code=-1;
		result.match=-1;
	 
		for(i=0;i<15;i++)
		{
		  result.match_vec[i]=0; 
		}
		return result;
	 }

 }
 
 /*void calc_per_col_div(int size, int length, FILE *fpN, int size_ext)
 {
 	int i=0,j=0;
 	int countA,countC,countG,countT,countN,countGAP,gcountN;
 	char tmpchar;
 	
 	gcountN = 0;
 	

 	for(i=0;i<length;i++)
 	{
 		countA=0;
 		countC=0;
 		countG=0;
 		countT=0;
 		countN=0;
 		countGAP=0;
 		
 		for(j=0;j<size_ext;j++)
 		{
			if(j<size)
			{
 				tmpchar = seq_d_table[j].sequence[i];
			}
			else
			{
 				tmpchar = seq_d_table_ext[j-size].sequence[i];

			}
 			
 			if(tmpchar=='A'){countA++;}
			if(tmpchar=='C'){countC++;}
			if(tmpchar=='G'){countG++;}
			if(tmpchar=='T' || tmpchar =='U'){countT++;}
			if(tmpchar=='N') // TODO
			{
				countN++; 
				gcountN++; 
			
				if(j<size)
				{
					store_N_position(gcountN,fpN,j,i);
				}
				else
				{
					fprintf(fpN, "\n\n   %d\t\t%s\n\t\t\t\t\t\t\t\t%d\n\t_____________________________________________________________________________________________________________________________\n", gcountN, seq_n_table_ext[j-size],i+1);
 				}
			}
			if(tmpchar=='-'){countGAP++;}
		 			
 		} 		 		

 	}
 	
 	fprintf(fpN, "\n\n\n\n   Total number of N characters in the alignment:\t\t%d\n\n",gcountN);
	ambtotal = ambtotal + gcountN;
	ambrtotal++;
 }*/

 void alter_N_report_path (char code)
 {
	 int i=0;

	 for(i=0;i<50;i++)
	 {
		 if((N_Report_path[i]=='_') && (N_Report_path[i+1]=='r') && (N_Report_path[i+2]=='e') && (N_Report_path[i+3]=='p') &&
			 (N_Report_path[i+4]=='o') && (N_Report_path[i+5]=='r') && (N_Report_path[i+6]=='t'))
			{
				break;
			}
	 }

	 N_Report_path[i-1] = code;

 }

 void report_label(int index,char code,FILE * fp, char *workspace,char * alignment)
 {
	 if(index==1)
	 {	
		fprintf(fp,"\n\n   ===========================\n");
		fprintf(fp,"    ChromatoGate ACD %c Report\n   ===========================\n\n\n",code);
		fprintf(fp,"   Workspace: \t\t%s\n\n",workspace);
		fprintf(fp,"   Alignment: \t\t%s\n\n\n\n",alignment);
	 
	 }
		
	 if(index==2)
	 {
		fprintf(fp,"\n\n   =========================\n");
		fprintf(fp,"    ChromatoGate PSD Report\n   =========================\n\n\n");
		fprintf(fp,"   Workspace: \t\t%s\n\n",workspace);
		fprintf(fp,"   Alignment: \t\t%s\n\n",alignment);
		fprintf(fp,"   Sensitivity: \t%d\n\n",diversity_thres);
		if(PSDsearch==1)
		{		
			fprintf(fp,"   Strategy: \t\tLE\n\n\n\n");
		}
		else
		{
			fprintf(fp,"   Strategy: \t\t1MC\n\n\n\n");
		}
	 }
	
	 if(index==3)
	 {
		fprintf(fp,"\n\n   ==================================\n");
		fprintf(fp,"    ChromatoGate post-PSD ACD Report\n   ==================================\n\n\n");
		fprintf(fp,"   Workspace: \t\t%s\n\n",workspace);
		fprintf(fp,"   Alignment: \t\t%s\n\n\n\n",alignment);
	 }

 }

 void calc_per_col_div_ambig(char ambiguity_code, int size, int length, int size_ext)
 {
 	int i=0,j=0;
 	int countA,countC,countG,countT,countN,countGAP,gcountN;
 	char tmpchar;
	FILE *fpN=NULL;
	int call=-1;
 	
 	gcountN = 0;

 	for(i=0;i<length;i++)
 	{
 		countA=0;
 		countC=0;
 		countG=0;
 		countT=0;
 		countN=0;
 		countGAP=0;
 		
 		for(j=0;j<size_ext;j++)
 		{
			if(j<size)
			{
 				tmpchar = seq_d_table[j].sequence[i];
			}
			else
			{
 				tmpchar = seq_d_table_ext[j-size].sequence[i];
			}
 			
 			if(tmpchar=='A'){countA++;}
			if(tmpchar=='C'){countC++;}
			if(tmpchar=='G'){countG++;}
			if(tmpchar=='T' || tmpchar=='U'){countT++;}
			if(tmpchar==ambiguity_code)
			{
				
				if (call==-1)
				{
					alter_N_report_path(ambiguity_code);

					fpN= fopen (N_Report_path,"w");	
 		
	    			report_label(1,ambiguity_code,fpN,workspace_name_g,ma1_filename);

					fprintf(fpN, "\n\t	SEQUENCE NAME \t\t\t\t ALIGNMENT SITE \t\t CHROMATOGRAM POSITION \t\t CHANGED TO \n   __________________________________________________________________________________________________________________________________ \n");
 		
					call=1;
				}
				countN++; 
				gcountN++; 
				
				if(j<size)
				{
					store_N_position(gcountN,fpN,j,i);
				}
				else
				{
					fprintf(fpN, "\n\n   %d\t\t%s\n\t\t\t\t\t\t\t\t%d\n\t_____________________________________________________________________________________________________________________________\n", gcountN, seq_n_table_ext[j-size],i+1);
				}
			}
			if(tmpchar=='-'){countGAP++;}
		 			
 		} 		 		

 	}
 	
	if (fpN!=NULL)
	{
 		fprintf(fpN, "\n\n\n\n   Total number of %c characters in the alignment:\t\t%d\n\n",ambiguity_code,gcountN);
		ambtotal = ambtotal + gcountN;		 
		ambrtotal++;
		fclose(fpN);
	}
 }
 
 void clear_seq(char * input)
 {
 	int i=0;
 	
 	for (i=0;i<1600;i++)
 	{
 		input[i]=0;
 	}
 }

 int char_compare (char ma1char, char ma2char)
 {
	 int result=-1;

	 if(ma1char==ma2char)
	 {
		 result=1;
	 }
	 else
	 {
		 result=0;
	 }

	 return result;
 }
 
 int get_chromatogram_position(int i, int j, int length_old, int mode)
 {
 	int chromatogram_position=-1, chromatogram_position_FW, chromatogram_position_RV;
	int testval1=0,testval2=0;
	int match_y=-1;
	char done;
 	
 	int num_of_removed_gaps=-1,sequence2_position=-1,degap_ind=-1,l=-1;
 	char tmpchar, char_ma1,char_ma2;
 	char degapped_seq_ma1[1600];
 	char degapped_seq_ma2[1600];
 	
 	int run_seq_ma1_index=0;
 	int run_seq_ma2_index=0;	
 	
 	num_of_removed_gaps = remove_gaps (i, seq_d_table[j].sequence2);
					
	sequence2_position = i - num_of_removed_gaps;

	degap_ind = 0;				
	clear_seq(degapped_seq_ma1);	
	
	for(l=0;l<length_old;l++)
	{
		tmpchar = seq_d_table[j].sequence[l];
	
		if (tmpchar != '-')
		{ 
			degapped_seq_ma1[degap_ind++]= tmpchar;
			testval1++;
		}

	}
	
	degap_ind = 0;				
	clear_seq(degapped_seq_ma2);
	
	for(l=0;l<ma2_length;l++)
	{
		tmpchar = seq_d_table[j].sequence2[l];

		if (tmpchar != '-')
		{ 			
			degapped_seq_ma2[degap_ind++]= tmpchar;
			testval2++;

		}

	}

	run_seq_ma1_index=-1;
	run_seq_ma2_index=-1;

	while(run_seq_ma2_index!=sequence2_position)
	{
		if((run_seq_ma1_index>length_old)||(run_seq_ma2_index>ma2_length))
		{
			/* BUG HERE*/
			printf("\n\n\n\n   => Inconsistency between the ambiguity reports and the alignment!\n\n", sequence2_position);
			printf("      Error detected in alignment site: %d \n\n",i,sequence2_position+1);
			printf("                             character: %c \n\n",degapped_seq_ma2[sequence2_position-1]);
			printf("                              sequence: %s \n\n\n",seq_n_table[j]);
			printf("      Update report(s) or alignment and try again.\n\n");

			printf("\n  Press Enter to exit...\n\n");
			scanf("%c",&done);scanf("%c",&done);

			exit(0); 
		}

		char_ma1=degapped_seq_ma1[run_seq_ma1_index+1];
		char_ma2=degapped_seq_ma2[run_seq_ma2_index+1];

		match_y=char_compare(char_ma1, char_ma2);
	
		if(match_y==1)
		{
			run_seq_ma1_index++;
			run_seq_ma2_index++;
		}
		else
		{
			run_seq_ma1_index++;
		}
	
	}

	if (seq_d_table[j].F_or_R == 0)
 	{
 		chromatogram_position = run_seq_ma1_index + seq_d_table[j].st_pos_chrom;
 	}
 	else
 	{	
		if (seq_d_table[j].F_or_R == 1)
		{
 			chromatogram_position = seq_d_table[j].la_pos_chrom - run_seq_ma1_index;
		}
		else
		{
			chromatogram_position_FW = get_cons_chrom_pos(j,run_seq_ma1_index,0);
			chromatogram_position_RV = get_cons_chrom_pos(j,run_seq_ma1_index,1);
		}
 	}

	if(mode==-1)
	{
		return chromatogram_position;
	}
	else
	{
		if(mode==0)
		{
			return chromatogram_position_FW;
		}
		else
		{
			return chromatogram_position_RV;
		}
	}
 	
	return -1;
 }
 
 char chromatogram_nucleotide(int sel, char nucleotide)
 {
	char result= nucleotide;

	if(sel==0)
	{
		result=nucleotide;
	}
	else
	{
		if(nucleotide=='A'){result='T';}
		if(nucleotide=='C'){result='G';}
		if(nucleotide=='G'){result='C';}
		if(nucleotide=='T' || nucleotide =='U'){result='A';}
		if(nucleotide=='N'){result='N';}
		if(nucleotide=='K'){result='M';}
		if(nucleotide=='M'){result='K';}
		if(nucleotide=='R'){result='Y';}
		if(nucleotide=='S'){result='S';}
		if(nucleotide=='W'){result='W';}
		if(nucleotide=='B'){result='V';}
		if(nucleotide=='H'){result='D';}
		if(nucleotide=='D'){result='H';}
	}

	return result;
 }
 void nucleotide_report(int flag,int i,int size, char nucleotide, int counter, FILE *fp, int length_old, int size_ext)
 {
 	int j=0;
 	int tcount=1;
 	char tmpchar,tmpchar2;
 	int sequence2_position=-1, num_of_removed_gaps=-1;
 	char temp_seq[1600];
 	char degapped_seq_ma1[1600];
 	char degapped_seq_ma2[1600];
 	int l=0, degap_ind =0, match=-1;
 	int chromatogram_position=-1, chromatogram_position_fw, chromatogram_position_rv;
 	int gaps,Lgaps,cons_code_pos;
 	FILE * fperror;

 	fprintf(fp,"\n\t\t\t\t\t     %c: %d\t\t",nucleotide,counter);
 	
 	if(flag==1)
 	{
			
	 	for(j=0;j<size_ext;j++)
	 	{
			if(j<size)
			{
				tmpchar = seq_d_table[j].sequence2[i];
			}
			else
			{
				tmpchar = seq_d_table_ext[j-size].sequence2[i];
			}

			if(tmpchar==nucleotide)
			{
				if(tcount==counter)
				{
					if(j<size)
					{
						fprintf(fp,"(%d): %s \t\t\t\t",tcount++,seq_n_table[j]);
					}
					else
					{
						fprintf(fp,"(%d): %s \t\t\t\t",tcount++,seq_n_table_ext[j-size]);
					}
				}
				else
				{
					if(j<size)
					{
						fprintf(fp,"(%d): %s , ",tcount++,seq_n_table[j]);
					}
					else
					{
						fprintf(fp,"(%d): %s , ",tcount++,seq_n_table_ext[j-size]);
					}
				}
			}
		}

		tcount=1;

		for(j=0;j<size;j++)
		{
			tmpchar = seq_d_table[j].sequence2[i];

			if(tmpchar==nucleotide)
			{				
				if(seq_d_table[j].F_or_R==0 || seq_d_table[j].F_or_R==1)
				{
					chromatogram_position = get_chromatogram_position(i,j, length_old,-1);								
							
					if(tcount==counter)
					{
						if(seq_d_table[j].F_or_R==0)
						{
							fprintf(fp,"(%d): FW.%d.%c",tcount++, chromatogram_position,chromatogram_nucleotide(seq_d_table[j].F_or_R,nucleotide));
						}
						if(seq_d_table[j].F_or_R==1)
						{
							fprintf(fp,"(%d): RV.%d.%c",tcount++, chromatogram_position,chromatogram_nucleotide(seq_d_table[j].F_or_R,nucleotide));
						}
					}
					else
					{
						if(seq_d_table[j].F_or_R==0)
						{
							fprintf(fp,"(%d): FW.%d.%c , ",tcount++, chromatogram_position,chromatogram_nucleotide(seq_d_table[j].F_or_R,nucleotide));
						}
						if(seq_d_table[j].F_or_R==1)
						{
							fprintf(fp,"(%d): RV.%d.%c , ",tcount++, chromatogram_position,chromatogram_nucleotide(seq_d_table[j].F_or_R,nucleotide));
						}
					}
				}
				else // case: consensus
				{
					gaps=remove_gaps(i,seq_d_table[j].sequence2);
					
					cons_code_pos = i - gaps;

					clear_seq(degapped_seq_ma1);	
					
					degap_ind=0;
					for(l=0;l<(int)strlen(seq_d_table[j].sequence);l++)
					{
						tmpchar = seq_d_table[j].sequence[l];
	
						if (tmpchar != '-')
						{ 
							degapped_seq_ma1[degap_ind++]= tmpchar;
						}

					}

					Lgaps = count_Lgaps(cons_code_pos,degapped_seq_ma1);

					cons_code_pos = cons_code_pos + Lgaps;

					chromatogram_position_fw = get_cons_chrom_pos_n(j,i,0,cons_code_pos);//get_chromatogram_position(i,j, length_old,0);								
					chromatogram_position_rv = get_cons_chrom_pos_n(j,i,1,cons_code_pos);//get_chromatogram_position(i,j, length_old,1);

					if(tcount==counter)
					{
						if(chromatogram_position_fw!=0 && chromatogram_position_rv!=0)
						{
							fprintf(fp,"(%d): FW.%d.%c - RV.%d.%c",tcount++, chromatogram_position_fw,chromatogram_nucleotide(0,nucleotide),chromatogram_position_rv,chromatogram_nucleotide(1,nucleotide));
						}
						else
						{
							if(chromatogram_position_fw==0)
							{
								fprintf(fp,"(%d): RV.%d.%c",tcount++, chromatogram_position_rv,chromatogram_nucleotide(1,nucleotide));
							}
							else
							{
								fprintf(fp,"(%d): FW.%d.%c",tcount++, chromatogram_position_fw,chromatogram_nucleotide(0,nucleotide));
							}
						}
						
					}
					else
					{
						if(chromatogram_position_fw!=0 && chromatogram_position_rv!=0)
						{
							fprintf(fp,"(%d): FW.%d.%c - RV.%d.%c, ",tcount++, chromatogram_position_fw,chromatogram_nucleotide(0,nucleotide),chromatogram_position_rv,chromatogram_nucleotide(1,nucleotide));
						}
						else
						{
							if(chromatogram_position_fw==0)
							{
								fprintf(fp,"(%d): RV.%d.%c, ",tcount++, chromatogram_position_rv,chromatogram_nucleotide(1,nucleotide));
							}
							else
							{
								fprintf(fp,"(%d): FW.%d.%c, ",tcount++, chromatogram_position_fw,chromatogram_nucleotide(0,nucleotide));
							}
						}
					}
				}
			}
		}
	}	
 }
 
 void diversity_report(int i, int domination_code, int * diversity_code, int *countv, int size, FILE *fp, int length_old, int size_ext )
 {
 	char nucleotide;
 	int counter=-1;
	char ambc[11]={'N','M','R','W','S','Y','K','V','H','D','B'};

	int j;

	char select_T_U;

	if (U_case==1)
	{
		select_T_U='U';
	}
	else
	{
		select_T_U='T';
	} 	
	
 	if(domination_code>0)
 	{

		nucleotide_report(diversity_code[0],i,size,'A',countv[0],fp,length_old,size_ext);
	 	nucleotide_report(diversity_code[1],i,size,'C',countv[1],fp,length_old,size_ext);
	 	nucleotide_report(diversity_code[2],i,size,'G',countv[2],fp,length_old,size_ext);
	 	nucleotide_report(diversity_code[3],i,size,select_T_U,countv[3],fp,length_old,size_ext);

		if(diversity_code[0]+diversity_code[1]+diversity_code[2]+diversity_code[3]==0)
		{
			for(j=0;j<11;j++)
			{	
				
				if(diversity_code[j+4]!=0)
				{
					nucleotide_report(diversity_code[j+4],i,size,ambc[j],countv[j+4],fp,length_old,size_ext);
				}

			}		
		}



		

		/*
		// Column dominated by Adenines 	
	 	if(domination_code==1)
	 	{	
	 		nucleotide_report(0,i,size,'A',countA,fp,length_old,size_ext);

	 		if(diversity_code==100)
	 		{	
	 			nucleotide_report(1,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,select_T_U,countT,fp,length_old,size_ext);
	 		}
	 		if(diversity_code==110)
	 		{
	 			nucleotide_report(1,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,select_T_U,countT,fp,length_old,size_ext);		
	 		}
	 		if(diversity_code==10)
	 		{
	 			nucleotide_report(0,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,select_T_U,countT,fp,length_old,size_ext);
	 		}
	 		if(diversity_code==111)
	 		{
	 	        nucleotide_report(1,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,select_T_U,countT,fp,length_old,size_ext);
	 			
	 		}
	 		if(diversity_code==101)
	 		{
	 	        nucleotide_report(1,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,select_T_U,countT,fp,length_old,size_ext);
	 			
	 		}
	 		if(diversity_code==11)
	 		{
				nucleotide_report(0,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,select_T_U,countT,fp,length_old,size_ext);
	 			
	 		}
	 		if(diversity_code==1)
	 		{
	 			nucleotide_report(0,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,select_T_U,countT,fp,length_old,size_ext);
	 		}
	 	
	 	}
	
		// Column dominated by Cytosines
		if(domination_code==2)
	 	{	
	 		
	 		if(diversity_code==1000)
	 		{	
	 			nucleotide_report(1,i,size,'A',countA,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,select_T_U,countT,fp,length_old,size_ext);
	 		}
	 		if(diversity_code==1010)
	 		{
	 			nucleotide_report(1,i,size,'A',countA,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,select_T_U,countT,fp,length_old,size_ext);		
	 		}
	 		if(diversity_code==10)
	 		{
	 			nucleotide_report(0,i,size,'A',countA,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,select_T_U,countT,fp,length_old,size_ext);
	 		}
	 		if(diversity_code==1011)
	 		{
	 			nucleotide_report(1,i,size,'A',countA,fp,length_old,size_ext);
	 	        nucleotide_report(0,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,select_T_U,countT,fp,length_old,size_ext);
	 			
	 		}
	 		if(diversity_code==11)
	 		{
	 			nucleotide_report(0,i,size,'A',countA,fp,length_old,size_ext);
				nucleotide_report(0,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,select_T_U,countT,fp,length_old,size_ext);
	 			
	 		}
	 		if(diversity_code==1)
	 		{
	 			nucleotide_report(0,i,size,'A',countA,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,select_T_U,countT,fp,length_old,size_ext);
	 		}
	 		if(diversity_code==1001)
	 		{
	 			nucleotide_report(1,i,size,'A',countA,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,select_T_U,countT,fp,length_old,size_ext);
	 		}
 	
	 	}
 	
	 	// Column dominated by Guanines
	 	if(domination_code==3)
	 	{
	 		
	 		if(diversity_code==1000)
	 		{	
	 			nucleotide_report(1,i,size,'A',countA,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,select_T_U,countT,fp,length_old,size_ext);
	 		}
	 		if(diversity_code==1100)
	 		{
	 			nucleotide_report(1,i,size,'A',countA,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,select_T_U,countT,fp,length_old,size_ext);		
	 		}
	 		if(diversity_code==100)
	 		{
	 			nucleotide_report(0,i,size,'A',countA,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,select_T_U,countT,fp,length_old,size_ext);
	 		}
	 		if(diversity_code==1101)
	 		{
	 			nucleotide_report(1,i,size,'A',countA,fp,length_old,size_ext);
	 	        nucleotide_report(1,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,select_T_U,countT,fp,length_old,size_ext);
	 			
	 		}
	 		if(diversity_code==101)
	 		{
	 			nucleotide_report(0,i,size,'A',countA,fp,length_old,size_ext);
				nucleotide_report(1,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,select_T_U,countT,fp,length_old,size_ext);
	 			
	 		}
	 		if(diversity_code==1)
	 		{
	 			nucleotide_report(0,i,size,'A',countA,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,select_T_U,countT,fp,length_old,size_ext);
	 		}
	 		if(diversity_code==1001)
	 		{
	 			nucleotide_report(1,i,size,'A',countA,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,select_T_U,countT,fp,length_old,size_ext);
	 		}	 	
	 	}
 	
	 	// Column dominated by Thymines
	 	if(domination_code==4)
	 	{
	 		
	 		if(diversity_code==1000)
	 		{	
	 			nucleotide_report(1,i,size,'A',countA,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,select_T_U,countT,fp,length_old,size_ext);
	 		}
	 		if(diversity_code==1100)
	 		{
	 			nucleotide_report(1,i,size,'A',countA,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,select_T_U,countT,fp,length_old,size_ext);		
	 		}
	 		if(diversity_code==100)
	 		{
	 			nucleotide_report(0,i,size,'A',countA,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,select_T_U,countT,fp,length_old,size_ext);
	 		}
	 		if(diversity_code==1110)
	 		{
	 			nucleotide_report(1,i,size,'A',countA,fp,length_old,size_ext);
	 	        nucleotide_report(1,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,select_T_U,countT,fp,length_old,size_ext);
	 			
	 		}
	 		if(diversity_code==110)
	 		{
	 			nucleotide_report(0,i,size,'A',countA,fp,length_old,size_ext);
				nucleotide_report(1,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,select_T_U,countT,fp,length_old,size_ext);
	 			
	 		}
	 		if(diversity_code==10)
	 		{
	 			nucleotide_report(0,i,size,'A',countA,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,select_T_U,countT,fp,length_old,size_ext);
	 		}
	 		if(diversity_code==1010)
	 		{
	 			nucleotide_report(1,i,size,'A',countA,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,select_T_U,countT,fp,length_old,size_ext);
	 		}
	 	
	 	}
	 	
	 	// Column dominated by Gaps
	 	if(domination_code==5)
	 	{
	 		
	 		if(diversity_code==1)
	 		{	
	 			nucleotide_report(0,i,size,'A',countA,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,select_T_U,countT,fp,length_old,size_ext);
	 		}
			if(diversity_code==10)
	 		{	
	 			nucleotide_report(0,i,size,'A',countA,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,select_T_U,countT,fp,length_old,size_ext);
	 		}
	 		if(diversity_code==11)
	 		{	
	 			nucleotide_report(0,i,size,'A',countA,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,select_T_U,countT,fp,length_old,size_ext);
	 		}
	 		if(diversity_code==100)
	 		{	
	 			nucleotide_report(0,i,size,'A',countA,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,select_T_U,countT,fp,length_old,size_ext);
	 		}
	 		if(diversity_code==101)
	 		{	
	 			nucleotide_report(0,i,size,'A',countA,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,select_T_U,countT,fp,length_old,size_ext);
	 		}
	 		if(diversity_code==110)
	 		{	
	 			nucleotide_report(0,i,size,'A',countA,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,select_T_U,countT,fp,length_old,size_ext);
	 		}	 		
	 		if(diversity_code==111)
	 		{	
	 			nucleotide_report(0,i,size,'A',countA,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,select_T_U,countT,fp,length_old,size_ext);
	 		}
	 		if(diversity_code==1000)
	 		{	
	 			nucleotide_report(1,i,size,'A',countA,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,select_T_U,countT,fp,length_old,size_ext);
	 		}
	 		if(diversity_code==1001)
	 		{	
	 			nucleotide_report(1,i,size,'A',countA,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,select_T_U,countT,fp,length_old,size_ext);
	 		}
	 		if(diversity_code==1010)
	 		{	
	 			nucleotide_report(1,i,size,'A',countA,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,select_T_U,countT,fp,length_old,size_ext);
	 		}
	 		if(diversity_code==1011)
	 		{	
	 			nucleotide_report(1,i,size,'A',countA,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,select_T_U,countT,fp,length_old,size_ext);
	 		}
	 		if(diversity_code==1100)
	 		{	
	 			nucleotide_report(1,i,size,'A',countA,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,select_T_U,countT,fp,length_old,size_ext);
	 		}
	 		if(diversity_code==1101)
	 		{	
	 			nucleotide_report(1,i,size,'A',countA,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,select_T_U,countT,fp,length_old,size_ext);
	 		}
	 		if(diversity_code==1110)
	 		{	
	 			nucleotide_report(1,i,size,'A',countA,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(0,i,size,select_T_U,countT,fp,length_old,size_ext);
	 		}
	 		if(diversity_code==1111)
	 		{	
	 			nucleotide_report(1,i,size,'A',countA,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'C',countC,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,'G',countG,fp,length_old,size_ext);
	 			nucleotide_report(1,i,size,select_T_U,countT,fp,length_old,size_ext);
	 		}	 	
	 	}
		*/
 	} 
 }

 int ambiguity_report_line(int entry_ind,int count, char ambiguity, int size_ext, int size, int column, FILE *fpA)
 {
	int j;
	char tmpchar;
	int entry_ind_loc = entry_ind;

	if(count>0)
	{
		for(j=0;j<size_ext;j++)
	 	{
			if(j<size)
			{
				tmpchar = seq_d_table[j].sequence2[column-1];
			}
			else
			{
				tmpchar = seq_d_table_ext[j-size].sequence2[column-1];
			}

			if(tmpchar==ambiguity)
			{
				
				if(entry_ind_loc==1)
				{
					entry_ind_loc++;
				}
				else
				{
					fprintf(fpA,"\n\t\t\t\t");
				}
				fprintf(fpA,"\t\t%c\t\t\t", ambiguity );

				if(j<size)
				{
					fprintf(fpA," %s \t\t\t\t",seq_n_table[j]);
				}
				else
				{
					fprintf(fpA," %s \t\t\t\t",seq_n_table_ext[j-size]);
				}
			}		
		}
	}

	return entry_ind_loc;
 }

 void ambiguity_report_total(int size,int size_ext,FILE *fpA, int amb_ind, int column, int countN, int countM, int countR, int countW, int countS, int countY, int countK, int countV, int countH, int countD, int countB)
 {
	int j=0, entry_ind=1, countval=0;
	char tmpchar,ambiguity;
    fprintf(fpA,"\n\n   %d \t\t\t%d\t", amb_ind, column );

	entry_ind=1;

	ambiguity = 'N';
	countval=countN;
	entry_ind=ambiguity_report_line(entry_ind,countval,ambiguity,size_ext,size,column,fpA);

	ambiguity = 'M';
	countval=countM;
	entry_ind=ambiguity_report_line(entry_ind,countval,ambiguity,size_ext,size,column,fpA);

	ambiguity = 'R';
	countval=countR;
	entry_ind=ambiguity_report_line(entry_ind,countval,ambiguity,size_ext,size,column,fpA);

	ambiguity = 'W';
	countval=countW;
	entry_ind=ambiguity_report_line(entry_ind,countval,ambiguity,size_ext,size,column,fpA);

	ambiguity = 'S';
	countval=countS;
	entry_ind=ambiguity_report_line(entry_ind,countval,ambiguity,size_ext,size,column,fpA);

	ambiguity = 'Y';
	countval=countY;
	entry_ind=ambiguity_report_line(entry_ind,countval,ambiguity,size_ext,size,column,fpA);

	ambiguity = 'K';
	countval=countK;
	entry_ind=ambiguity_report_line(entry_ind,countval,ambiguity,size_ext,size,column,fpA);

	ambiguity = 'V';
	countval=countV;
	entry_ind=ambiguity_report_line(entry_ind,countval,ambiguity,size_ext,size,column,fpA);

	ambiguity = 'H';
	countval=countH;
	entry_ind=ambiguity_report_line(entry_ind,countval,ambiguity,size_ext,size,column,fpA);

	ambiguity = 'D';
	countval=countD;
	entry_ind=ambiguity_report_line(entry_ind,countval,ambiguity,size_ext,size,column,fpA);

	ambiguity = 'B';
	countval=countB;
	entry_ind=ambiguity_report_line(entry_ind,countval,ambiguity,size_ext,size,column,fpA);
	
 	fprintf(fpA,"\n\t________________________________________________________________________________\n");
 	 	
 }
			
 int total_amb_g;
 int total_unalcol_g;

 void calc_per_col_div_MA2(int size, int length, FILE *fp, FILE *fpA, int length_old, int size_ext)
 {
 	int i=0,j=0, counter=1,l=0,amb_ind=0;
 	int countA,countC,countG,countT,countN,
		countM,countR,countW,countS,countY,
		countK,countV,countH,countD,countB,countGAP,match=-1, code=-1, amb_case=0, total_amb=0;
	int countv[15];
 	char tmpchar;
 	int tcount=1;
 	div_r div_rep;

	 	 		
 	for(i=0;i<length;i++)
 	{
 		countA=0;
 		countC=0;
 		countG=0;
 		countT=0;
 		countN=0;
		countM=0;
		countR=0;
		countW=0;
		countS=0;
		countY=0;
		countK=0;
		countV=0;
		countH=0;
		countD=0;
		countB=0;
		countGAP=0;
 		
 		for(j=0;j<size_ext;j++)
 		{
			if(j<size)
			{
 				tmpchar = seq_d_table[j].sequence2[i];
			}
			else
			{
 				tmpchar = seq_d_table_ext[j-size].sequence2[i];
			}

 			if(tmpchar=='A'){countA++;}
			if(tmpchar=='C'){countC++;}
			if(tmpchar=='G'){countG++;}
			if(tmpchar=='T' || tmpchar=='U'){countT++;}

			if(tmpchar=='N'){countN++;}
			if(tmpchar=='M'){countM++;}
			if(tmpchar=='R'){countR++;}
			if(tmpchar=='W'){countW++;}
			if(tmpchar=='S'){countS++;}
			if(tmpchar=='Y'){countY++;}
			if(tmpchar=='K'){countK++;}
			if(tmpchar=='V'){countV++;}
			if(tmpchar=='H'){countH++;}
			if(tmpchar=='D'){countD++;}
			if(tmpchar=='B'){countB++;}
						
			if(tmpchar=='-'){countGAP++;}
		 			
 		}

		countv[0]=countA;
		countv[1]=countC;
		countv[2]=countG;
		countv[3]=countT;
		countv[4]=countN;
		countv[5]=countM;
		countv[6]=countR;
		countv[7]=countW;
		countv[8]=countS;
		countv[9]=countY;
		countv[10]=countK;
		countv[11]=countV;
		countv[12]=countH;
		countv[13]=countD;
		countv[14]=countB;

 		div_rep.code=-1;
 		div_rep.match=-1;	
 		div_rep = diversity_check (countv,countA, countC, countG, countT, countN, countM, countR, countW, countS, countY, countK, countV, countH, countD, countB, countGAP);		 

		amb_case=0;
		amb_case= countN + countM + countR + countW + countS + countY + countK + countV + countH + countD + countB;

		if(amb_case>0)
		{
			total_amb = total_amb + amb_case;
			amb_ind++;
			ambiguity_report_total(size,size_ext,fpA,amb_ind, i+1, countN, countM, countR, countW, countS, countY, countK, countV, countH, countD, countB);
			
		}		

		if(div_rep.match>0)
 		{
 			fprintf(fp,"\n\n   %d \t\t\t%d\t", counter++, i+1 );

			diversity_report(i,div_rep.code, div_rep.match_vec, countv, size, fp, length_old, size_ext);
 		
 			fprintf(fp,"\n\t_____________________________________________________________________________________________________________________________\n");
 		} 		
 	}

	fprintf(fpA, "\n\n\n\n   Total number of ambiguous characters:\t\t%d\n\n",total_amb);

	total_amb_g = total_amb;

	fprintf(fp, "\n\n\n\n   Total number of polymorphic sites:\t\t%d\n\n",counter-1);

	total_unalcol_g = counter-1;
 }
 
 
 int match_seq_name(int size, char * tmpchar)
 {
 	int seq_index = -1;
 	int i=0;
 	
 	for(i=0;i<size;i++)
 	{
 		if(strcmp(tmpchar,seq_n_table[i])==0)
 		{
 			seq_index=i+1;
 			break;
 		}
 	}
 	 	
 	return seq_index;
 }
 
 void update_data(char code,int seq_index,int Nposition,char Nstatus)
 {
	char done;
 	
	//if (AMBIGUITY_ERROR_CHECK==1)
	//{
		if(Nstatus=='A' || Nstatus=='C' || Nstatus=='G' || Nstatus=='T' || Nstatus=='-' || Nstatus=='N' ||
		   Nstatus=='U' || Nstatus=='M' || Nstatus=='R' || Nstatus=='W' || Nstatus=='S' || Nstatus=='Y' ||
		   Nstatus=='K' || Nstatus=='V' || Nstatus=='H' || Nstatus=='D' || Nstatus=='B')
 		{

			seq_d_table[seq_index-1].sequence[Nposition-1]=Nstatus;

 			if(Nstatus=='-')
 			{
				seq_d_table[seq_index-1].sequence[Nposition-1]='L';
 			}
		}
		else
		{
			if(Nstatus=='?')
			{
				seq_d_table[seq_index-1].sequence[Nposition-1]=code;
			}
			else
			{
				printf("\n\n\n\n   => Invalid character (%c) has been detected in the %c_Report file.\n\n",Nstatus,code);
				printf("      Update the %c_Report and try again.\n\n",code);

				printf("\n  Press Enter to exit...\n\n");
				scanf("%c",&done);scanf("%c",&done);
		
				exit(0);
			}
		}		
 	/*}
 	else
 	{
		
		if(Nstatus=='?')
		{
			seq_d_table[seq_index-1].sequence[Nposition-1]=code;
		}
		else
		{
			printf("\n\n\n\n   => Invalid character (%c) has been detected in the %c_Report file.\n\n",Nstatus,code);
			printf("      Update the %c_Report and try again.\n\n",code);

			printf("\n  Press Enter to exit...\n\n");
			scanf("%c",&done);scanf("%c",&done);
		
			exit(0);
		}		
	}*/
 
 }

 int get_report_size(FILE * fpN)
 {
	 char tchar;
	 int size;

	tchar=fgetc(fpN); 	
 	while(tchar!=':'){tchar=fgetc(fpN);}
	tchar=fgetc(fpN);
	while(tchar!=':'){tchar=fgetc(fpN);}
	tchar=fgetc(fpN);
	fscanf(fpN,"%d",&size);

	while(tchar!=EOF)
	{
		tchar=fgetc(fpN);
		fscanf(fpN,"%d",&size);

	}

	return size;

 }
 void read_X_report(char code, int max_F_num, FILE *fpN)
 {
 	int stop=0;
 	char tmpchar[100],Nstatus;
	char trash[500];
 	int Nposition; 	
 	int size=0;
 	int i=0,Nval=-1,seq_index=-1;

 	size= get_report_size(fpN);

 	rewind(fpN);

	for(i=0;i<15;i++)
	{
		fscanf(fpN,"%s",trash);
	}
 	
 	tmpchar[0]=fgetc(fpN);
 	
 	while(tmpchar[0]!='1')
 	{
 		tmpchar[0]=fgetc(fpN); 		
 	} 	
 	 	
 	for(i=0;i<size;i++)
 	{
		tmpchar[0]=fgetc(fpN); tmpchar[0]=fgetc(fpN); 
 		fscanf(fpN,"%s",tmpchar);
 	
 		seq_index = match_seq_name(max_F_num,tmpchar);
 
  		
  		fscanf(fpN,"%d",&Nposition);


 		tmpchar[0]=fgetc(fpN);
 	
 		while(tmpchar[0]!='>')
 		{
 			tmpchar[0]=fgetc(fpN); 		
 		}
 		
 		fscanf(fpN,"%c",&Nstatus);
 		

 		update_data(code,seq_index,Nposition,Nstatus);
 		
 		if(i==size-1){break;}
 		else
		{
			tmpchar[0]=fgetc(fpN); tmpchar[0]=fgetc(fpN); 
			fscanf(fpN,"%s",trash); tmpchar[0]=fgetc(fpN); tmpchar[0]=fgetc(fpN); tmpchar[0]=fgetc(fpN); tmpchar[0]=fgetc(fpN); tmpchar[0]=fgetc(fpN);
			fscanf(fpN,"%d",&Nval);
		}
 	} 	
 	 	
 }
 
 void print_label()
 {
 	printf("\n\n\t\t___________________________________________________\n\n\t\t\t\t  ChromatoGate 1.2 \n\t\t___________________________________________________\n\n"); 	
 	printf("\n ChromatoGate 1.2 released by Nikolaos Ch. Alachiotis in October 2011.\n\n");
 }

 void create_consensus_folders(char * workspace_name)
 {
	char workspace[110];
 	int i=0;
 	int j=0;
 	
 	for(i=0;i<(int)strlen(workspace_name);i++)
 	{
 		workspace[i]=workspace_name[i];
		Consensus_Forward_Path[i]=workspace_name[i];
		Consensus_Reverse_Path[i]=workspace_name[i];
		consensus_out_file[i]=workspace_name[i];
		consensus_fas_file[i]=workspace_name[i];
		consensus_inspection_folder_path[i]=workspace_name[i];
 	
 	}


 	i=(int)strlen(workspace_name);
 	workspace[i++]='_';workspace[i++]='W';workspace[i++]='O';workspace[i++]='R';workspace[i++]='K';workspace[i++]='S';workspace[i++]='P';
 	workspace[i++]='A';workspace[i++]='C';workspace[i++]='E';
	workspace[i++]='/';
	workspace[i++]='C';workspace[i++]='O';workspace[i++]='N';workspace[i++]='S';workspace[i++]='E';workspace[i++]='N';workspace[i++]='S';
	workspace[i++]='U';workspace[i++]='S';workspace[i++]=0;
 	
	mkdir(workspace,0777); 

	i=(int)strlen(workspace_name);
 	consensus_inspection_folder_path[i++]='_';consensus_inspection_folder_path[i++]='W';consensus_inspection_folder_path[i++]='O';
	consensus_inspection_folder_path[i++]='R';consensus_inspection_folder_path[i++]='K';consensus_inspection_folder_path[i++]='S';
	consensus_inspection_folder_path[i++]='P';
 	consensus_inspection_folder_path[i++]='A';consensus_inspection_folder_path[i++]='C';consensus_inspection_folder_path[i++]='E';
	consensus_inspection_folder_path[i++]='/';
	consensus_inspection_folder_path[i++]='C';consensus_inspection_folder_path[i++]='O';consensus_inspection_folder_path[i++]='N';
	consensus_inspection_folder_path[i++]='S';consensus_inspection_folder_path[i++]='E';consensus_inspection_folder_path[i++]='N';
	consensus_inspection_folder_path[i++]='S';
	consensus_inspection_folder_path[i++]='U';consensus_inspection_folder_path[i++]='S';
	consensus_inspection_folder_path[i++]='/';
	consensus_inspection_folder_path[i++]='I';
	consensus_inspection_folder_path[i++]='N';consensus_inspection_folder_path[i++]='S';consensus_inspection_folder_path[i++]='P';
	consensus_inspection_folder_path[i++]='E';consensus_inspection_folder_path[i++]='C';consensus_inspection_folder_path[i++]='T';	
	consensus_inspection_folder_path[i++]=0;

	
 	i=(int)strlen(workspace_name);
 	workspace[i++]='_';workspace[i++]='W';workspace[i++]='O';workspace[i++]='R';workspace[i++]='K';workspace[i++]='S';workspace[i++]='P';
 	workspace[i++]='A';workspace[i++]='C';workspace[i++]='E';
	workspace[i++]='/';
	workspace[i++]='C';workspace[i++]='O';workspace[i++]='N';workspace[i++]='S';workspace[i++]='E';workspace[i++]='N';workspace[i++]='S';workspace[i++]='U';
	workspace[i++]='S';workspace[i++]='/';
	workspace[i++]='F';workspace[i++]='O';workspace[i++]='R';workspace[i++]='W';workspace[i++]='A';workspace[i++]='R';workspace[i++]='D';
 	workspace[i++]=0;
 	
	mkdir(workspace,0777); 	

	i=(int)strlen(workspace_name);
	while(workspace[i]!=0)
	{
		Consensus_Forward_Path[i]=workspace[i];
		Consensus_Forward_Path[i+1]=0;
		i++;
	}

	i=(int)strlen(workspace_name);
 	workspace[i++]='_';workspace[i++]='W';workspace[i++]='O';workspace[i++]='R';workspace[i++]='K';workspace[i++]='S';workspace[i++]='P';
 	workspace[i++]='A';workspace[i++]='C';workspace[i++]='E';
	workspace[i++]='/';
	workspace[i++]='C';workspace[i++]='O';workspace[i++]='N';workspace[i++]='S';workspace[i++]='E';workspace[i++]='N';workspace[i++]='S';workspace[i++]='U';
	workspace[i++]='S';workspace[i++]='/';
	workspace[i++]='R';workspace[i++]='E';workspace[i++]='V';workspace[i++]='E';workspace[i++]='R';workspace[i++]='S';
 	workspace[i++]='E';workspace[i++]=0;
 	
 	mkdir(workspace,0777); 	
 	
	i=(int)strlen(workspace_name);
	while(workspace[i]!=0)
	{
		Consensus_Reverse_Path[i]=workspace[i];
		Consensus_Reverse_Path[i+1]=0;
		i++;
	}

	i=(int)strlen(workspace_name);
 	consensus_out_file[i++]='_'; consensus_out_file[i++]='W'; consensus_out_file[i++]='O';
 	consensus_out_file[i++]='R'; consensus_out_file[i++]='K'; consensus_out_file[i++]='S';  consensus_out_file[i++]='P';
 	consensus_out_file[i++]='A'; consensus_out_file[i++]='C'; consensus_out_file[i++]='E'; 
 	consensus_out_file[i++]='/'; consensus_out_file[i++]='D'; consensus_out_file[i++]='A';  
 	consensus_out_file[i++]='T'; consensus_out_file[i++]='A'; consensus_out_file[i++]='/'; 
 	consensus_out_file[i++]='c'; consensus_out_file[i++]='o'; consensus_out_file[i++]='n';
 	consensus_out_file[i++]='s'; consensus_out_file[i++]='e'; consensus_out_file[i++]='n'; 	consensus_out_file[i++]='s';
 	consensus_out_file[i++]='u'; consensus_out_file[i++]='s'; consensus_out_file[i++]='.';  consensus_out_file[i++]='o';
 	consensus_out_file[i++]='u'; consensus_out_file[i++]='t'; consensus_out_file[i++]=0;
 	
	i=(int)strlen(workspace_name);
 	consensus_fas_file[i++]='_'; consensus_fas_file[i++]='W'; consensus_fas_file[i++]='O';
 	consensus_fas_file[i++]='R'; consensus_fas_file[i++]='K'; consensus_fas_file[i++]='S';  consensus_fas_file[i++]='P';
 	consensus_fas_file[i++]='A'; consensus_fas_file[i++]='C'; consensus_fas_file[i++]='E'; 
 	consensus_fas_file[i++]='/'; consensus_fas_file[i++]='C'; consensus_fas_file[i++]='O';  
 	consensus_fas_file[i++]='N'; consensus_fas_file[i++]='S'; consensus_fas_file[i++]='E'; 
 	consensus_fas_file[i++]='S'; consensus_fas_file[i++]='U'; consensus_fas_file[i++]='S';
 	consensus_fas_file[i++]='/'; consensus_fas_file[i++]='c'; consensus_fas_file[i++]='o'; 
	consensus_fas_file[i++]='n'; consensus_fas_file[i++]='s'; consensus_fas_file[i++]='e'; consensus_fas_file[i++]='n';
 	consensus_fas_file[i++]='s'; consensus_fas_file[i++]='u'; consensus_fas_file[i++]='s'; consensus_fas_file[i++]='.';  
 	consensus_fas_file[i++]='f'; consensus_fas_file[i++]='a'; consensus_fas_file[i++]='s'; consensus_fas_file[i++]=0; 	

 }
 
 void create_workspace(char * workspace_name)
 {
 	char workspace[110];
 	int i=0;
 	int j=0;
	int y=0;
	FILE * fp;
 	
 	for(i=0;i<(int)strlen(workspace_name);i++)
 	{
 		workspace[i]=workspace_name[i];
		workspace_name_g[i]=workspace[i];
 		pre_alignment_fas_path[i]=workspace_name[i];
 		pre_alignment_out_path[i]=workspace_name[i];
 		lut_out_path[i]=workspace_name[i];
		consensus_out_path[i]=workspace_name[i];
		chromatogram_out_path[i]=workspace_name[i];
 	
 	}
	workspace_name_g[i]=0;
 	
 	i=(int)strlen(workspace_name);
 	workspace[i++]='_';workspace[i++]='W';workspace[i++]='O';workspace[i++]='R';workspace[i++]='K';workspace[i++]='S';workspace[i++]='P';
 	workspace[i++]='A';workspace[i++]='C';workspace[i++]='E';workspace[i++]=0;
 	
 	mkdir(workspace,0777); 	
 	
 	i=(int)strlen(workspace_name);
 	pre_alignment_fas_path[i++]='_'; pre_alignment_fas_path[i++]='W'; pre_alignment_fas_path[i++]='O';
 	pre_alignment_fas_path[i++]='R'; pre_alignment_fas_path[i++]='K'; pre_alignment_fas_path[i++]='S';  pre_alignment_fas_path[i++]='P';
 	pre_alignment_fas_path[i++]='A'; pre_alignment_fas_path[i++]='C'; pre_alignment_fas_path[i++]='E'; 
 	pre_alignment_fas_path[i++]='/'; 
	
	y=0;
	for(j=i;j<(int)strlen(workspace_name)+i;j++)
	{
		pre_alignment_fas_path[j]=workspace_name[y]; 
		y++;
	}

	i=j;	
	
	pre_alignment_fas_path[i++]='_';
	pre_alignment_fas_path[i++]='p'; pre_alignment_fas_path[i++]='r'; 	pre_alignment_fas_path[i++]='e';
 	pre_alignment_fas_path[i++]='l'; pre_alignment_fas_path[i++]='i';
 	pre_alignment_fas_path[i++]='m'; pre_alignment_fas_path[i++]='i';pre_alignment_fas_path[i++]='n'; pre_alignment_fas_path[i++]='a';  pre_alignment_fas_path[i++]='r';
 	pre_alignment_fas_path[i++]='y';  pre_alignment_fas_path[i++]='.';  pre_alignment_fas_path[i++]='f';
 	pre_alignment_fas_path[i++]='a'; pre_alignment_fas_path[i++]='s'; pre_alignment_fas_path[i++]=0; 	
 	
 	
 	i=(int)strlen(workspace_name);
 	pre_alignment_out_path[i++]='_'; pre_alignment_out_path[i++]='W'; pre_alignment_out_path[i++]='O';
 	pre_alignment_out_path[i++]='R'; pre_alignment_out_path[i++]='K'; pre_alignment_out_path[i++]='S';  pre_alignment_out_path[i++]='P';
 	pre_alignment_out_path[i++]='A'; pre_alignment_out_path[i++]='C'; pre_alignment_out_path[i++]='E'; 
 	pre_alignment_out_path[i++]='/'; pre_alignment_out_path[i++]='D'; pre_alignment_out_path[i++]='A';  
 	pre_alignment_out_path[i++]='T'; pre_alignment_out_path[i++]='A'; pre_alignment_out_path[i++]='/'; 
 	pre_alignment_out_path[i++]='p'; pre_alignment_out_path[i++]='r'; pre_alignment_out_path[i++]='e';
 	pre_alignment_out_path[i++]='_'; pre_alignment_out_path[i++]='a'; pre_alignment_out_path[i++]='l'; 	pre_alignment_out_path[i++]='i';
 	pre_alignment_out_path[i++]='g'; pre_alignment_out_path[i++]='n'; pre_alignment_out_path[i++]='m';  pre_alignment_out_path[i++]='e';
 	pre_alignment_out_path[i++]='n'; pre_alignment_out_path[i++]='t'; pre_alignment_out_path[i++]='.';  pre_alignment_out_path[i++]='o';
 	pre_alignment_out_path[i++]='u'; pre_alignment_out_path[i++]='t'; pre_alignment_out_path[i++]=0;

 	
	i=(int)strlen(workspace_name);
 	consensus_out_path[i++]='_'; consensus_out_path[i++]='W'; consensus_out_path[i++]='O';
 	consensus_out_path[i++]='R'; consensus_out_path[i++]='K'; consensus_out_path[i++]='S'; consensus_out_path[i++]='P';
 	consensus_out_path[i++]='A'; consensus_out_path[i++]='C'; consensus_out_path[i++]='E'; 
 	consensus_out_path[i++]='/'; consensus_out_path[i++]='D'; consensus_out_path[i++]='A';  
 	consensus_out_path[i++]='T'; consensus_out_path[i++]='A'; consensus_out_path[i++]='/'; 
 	consensus_out_path[i++]='c'; consensus_out_path[i++]='o'; consensus_out_path[i++]='n';
 	consensus_out_path[i++]='s'; consensus_out_path[i++]='e'; consensus_out_path[i++]='n'; consensus_out_path[i++]='s';
 	consensus_out_path[i++]='u'; consensus_out_path[i++]='s'; consensus_out_path[i++]='.'; consensus_out_path[i++]='o';
 	consensus_out_path[i++]='u'; consensus_out_path[i++]='t'; consensus_out_path[i++]=0;


 	i=(int)strlen(workspace_name);
 	lut_out_path[i++]='_'; lut_out_path[i++]='W'; lut_out_path[i++]='O';
 	lut_out_path[i++]='R'; lut_out_path[i++]='K'; lut_out_path[i++]='S';  lut_out_path[i++]='P';
 	lut_out_path[i++]='A'; lut_out_path[i++]='C'; lut_out_path[i++]='E'; 
 	lut_out_path[i++]='/'; lut_out_path[i++]='D'; lut_out_path[i++]='A';  lut_out_path[i++]='T';
 	lut_out_path[i++]='A'; lut_out_path[i++]='/'; lut_out_path[i++]='l';  lut_out_path[i++]='u';
 	lut_out_path[i++]='t'; lut_out_path[i++]='.'; lut_out_path[i++]='o';  lut_out_path[i++]='u';
 	lut_out_path[i++]='t'; lut_out_path[i++]=0; 	
 	
 	
 	i=10+(int)strlen(workspace_name);
 	workspace[i++]='/';workspace[i++]='F';workspace[i++]='O';workspace[i++]='R';workspace[i++]='W';workspace[i++]='A';workspace[i++]='R';
 	workspace[i++]='D';workspace[i++]=0;


	i=(int)strlen(workspace_name);
 	chromatogram_out_path[i++]='_'; chromatogram_out_path[i++]='W'; chromatogram_out_path[i++]='O';
 	chromatogram_out_path[i++]='R'; chromatogram_out_path[i++]='K'; chromatogram_out_path[i++]='S';  chromatogram_out_path[i++]='P';
 	chromatogram_out_path[i++]='A'; chromatogram_out_path[i++]='C'; chromatogram_out_path[i++]='E'; 
 	chromatogram_out_path[i++]='/'; chromatogram_out_path[i++]='D'; chromatogram_out_path[i++]='A';  chromatogram_out_path[i++]='T';
 	chromatogram_out_path[i++]='A'; chromatogram_out_path[i++]='/'; chromatogram_out_path[i++]='c';  chromatogram_out_path[i++]='h';
 	chromatogram_out_path[i++]='r'; chromatogram_out_path[i++]='o'; chromatogram_out_path[i++]='m'; 
	chromatogram_out_path[i++]='a'; chromatogram_out_path[i++]='t'; chromatogram_out_path[i++]='o'; 
	chromatogram_out_path[i++]='g'; chromatogram_out_path[i++]='r'; chromatogram_out_path[i++]='a'; 
	chromatogram_out_path[i++]='m'; chromatogram_out_path[i++]='.'; chromatogram_out_path[i++]='o';  chromatogram_out_path[i++]='u';
 	chromatogram_out_path[i++]='t'; chromatogram_out_path[i++]=0; 

 	
 	for(j=0;j<=(int)strlen(workspace);j++)
 	{
 		FORWARD_path[j]=workspace[j];
 	} 	
 	
 	mkdir(workspace,0777);
 	
	i=10+(int)strlen(workspace_name);
 	workspace[i++]='/';workspace[i++]='R';workspace[i++]='E';workspace[i++]='V';workspace[i++]='E';workspace[i++]='R';workspace[i++]='S';
 	workspace[i++]='E';workspace[i++]=0;
 	
 	for(j=0;j<=(int)strlen(workspace);j++)
 	{
 		REVERSE_path[j]=workspace[j];
 	}
 	
 	mkdir(workspace,0777);	

	i=10+(int)strlen(workspace_name);
 	workspace[i++]='/';workspace[i++]='D';workspace[i++]='A';workspace[i++]='T';workspace[i++]='A';workspace[i++]=0;
 	
 	mkdir(workspace,0777);
 	
 }
 
 void create_path(int path_index,char * tmp, char * workspace_name)
 {
 	
 	int i=0,j=0,k=0;
 	
 	if (path_index==AMBIGUITIES_RUN)
 	{
 			 	
	 	for(i=0;i<(int)strlen(workspace_name);i++)
	 	{
	 		ma1_name[i]=workspace_name[i];
	 		N_Report_path[i]=workspace_name[i];
	 		ma1_file_path[i]=workspace_name[i];
	  	}	
	 	
	 	i=(int)strlen(workspace_name);
	 	ma1_name[i++]='_'; ma1_name[i++]='W'; ma1_name[i++]='O'; ma1_name[i++]='R'; ma1_name[i++]='K'; ma1_name[i++]='S'; ma1_name[i++]='P';
	 	ma1_name[i++]='A'; ma1_name[i++]='C'; ma1_name[i++]='E'; ma1_name[i++]='/';	 	
	 	
	 	
	 	for(j=0;j<(int)strlen(tmp);j++)
	 	{
			ma1_filename[k++]=tmp[j];
	 		ma1_name[i++]=tmp[j];
	  	}
		ma1_filename[k]=0;
	 	ma1_name[i++]=0;
	 	
	 	i=(int)strlen(workspace_name);
	 	N_Report_path[i++]='_'; N_Report_path[i++]='W'; N_Report_path[i++]='O'; N_Report_path[i++]='R'; N_Report_path[i++]='K'; 
	 	N_Report_path[i++]='S'; N_Report_path[i++]='P'; N_Report_path[i++]='A'; N_Report_path[i++]='C'; N_Report_path[i++]='E'; 
	 	N_Report_path[i++]='/'; N_Report_path[i++]='N'; N_Report_path[i++]='_'; N_Report_path[i++]='r'; N_Report_path[i++]='e';
	 	N_Report_path[i++]='p'; N_Report_path[i++]='o'; N_Report_path[i++]='r'; N_Report_path[i++]='t'; N_Report_path[i++]='.';
	 	N_Report_path[i++]='t'; N_Report_path[i++]='x'; N_Report_path[i++]='t'; N_Report_path[i++]=0;
 		
 		i=(int)strlen(workspace_name);
	 	ma1_file_path[i++]='_'; ma1_file_path[i++]='W'; ma1_file_path[i++]='O'; ma1_file_path[i++]='R'; ma1_file_path[i++]='K'; 
	 	ma1_file_path[i++]='S'; ma1_file_path[i++]='P'; ma1_file_path[i++]='A'; ma1_file_path[i++]='C'; ma1_file_path[i++]='E'; 
	 	ma1_file_path[i++]='/'; ma1_file_path[i++]='D'; ma1_file_path[i++]='A'; ma1_file_path[i++]='T'; ma1_file_path[i++]='A';
	 	ma1_file_path[i++]='/'; ma1_file_path[i++]='m'; ma1_file_path[i++]='a'; ma1_file_path[i++]='1'; ma1_file_path[i++]='_';
	 	ma1_file_path[i++]='p'; ma1_file_path[i++]='a'; ma1_file_path[i++]='t'; ma1_file_path[i++]='h'; ma1_file_path[i++]='.';
	 	ma1_file_path[i++]='o'; ma1_file_path[i++]='u'; ma1_file_path[i++]='t';
 	
 	}
 	
 	if (path_index==DIVERSITY_RUN)
 	{
 			 	
	 	for(i=0;i<(int)strlen(workspace_name);i++)
	 	{
	 		ma2_name[i]=workspace_name[i];
	 		N_Report_path[i]=workspace_name[i];
	 		D_Report_path[i]=workspace_name[i];
			A_Report_path[i]=workspace_name[i];
	 		ma1_file_path[i]=workspace_name[i];
	  	}	
	 	
	 	i=(int)strlen(workspace_name);
	 	ma2_name[i++]='_'; ma2_name[i++]='W'; ma2_name[i++]='O'; ma2_name[i++]='R'; ma2_name[i++]='K'; ma2_name[i++]='S'; ma2_name[i++]='P';
	 	ma2_name[i++]='A'; ma2_name[i++]='C'; ma2_name[i++]='E'; ma2_name[i++]='/';	 	
	 	
	 	for(j=0;j<(int)strlen(tmp);j++)
	 	{
			ma2_filename[k++]=tmp[j];
	 		ma2_name[i++]=tmp[j];
	  	}
		ma2_filename[k]=0;
	 	ma2_name[i++]=0;	 	

	 	i=(int)strlen(workspace_name);
	 	N_Report_path[i++]='_'; N_Report_path[i++]='W'; N_Report_path[i++]='O'; N_Report_path[i++]='R'; N_Report_path[i++]='K'; 
	 	N_Report_path[i++]='S'; N_Report_path[i++]='P'; N_Report_path[i++]='A'; N_Report_path[i++]='C'; N_Report_path[i++]='E'; 
	 	N_Report_path[i++]='/'; N_Report_path[i++]='N'; N_Report_path[i++]='_'; N_Report_path[i++]='r'; N_Report_path[i++]='e';
	 	N_Report_path[i++]='p'; N_Report_path[i++]='o'; N_Report_path[i++]='r'; N_Report_path[i++]='t'; N_Report_path[i++]='.';
	 	N_Report_path[i++]='t'; N_Report_path[i++]='x'; N_Report_path[i++]='t'; N_Report_path[i++]=0;
 		
 		i=(int)strlen(workspace_name);
	 	D_Report_path[i++]='_'; D_Report_path[i++]='W'; D_Report_path[i++]='O'; D_Report_path[i++]='R'; D_Report_path[i++]='K'; 
	 	D_Report_path[i++]='S'; D_Report_path[i++]='P'; D_Report_path[i++]='A'; D_Report_path[i++]='C'; D_Report_path[i++]='E'; 
	 	D_Report_path[i++]='/'; D_Report_path[i++]='P'; D_Report_path[i++]='S'; D_Report_path[i++]='D'; D_Report_path[i++]='_'; 
	 	D_Report_path[i++]='r'; D_Report_path[i++]='e'; D_Report_path[i++]='p'; D_Report_path[i++]='o'; D_Report_path[i++]='r'; 
	 	D_Report_path[i++]='t'; D_Report_path[i++]='.'; D_Report_path[i++]='t'; D_Report_path[i++]='x'; D_Report_path[i++]='t'; 
	 	D_Report_path[i++]=0;
	 	
		i=(int)strlen(workspace_name);
	 	A_Report_path[i++]='_'; A_Report_path[i++]='W'; A_Report_path[i++]='O'; A_Report_path[i++]='R'; A_Report_path[i++]='K'; 
	 	A_Report_path[i++]='S'; A_Report_path[i++]='P'; A_Report_path[i++]='A'; A_Report_path[i++]='C'; A_Report_path[i++]='E'; 
	 	A_Report_path[i++]='/'; A_Report_path[i++]='A'; A_Report_path[i++]='C'; A_Report_path[i++]='D'; A_Report_path[i++]='_'; 
	 	A_Report_path[i++]='r'; A_Report_path[i++]='e'; A_Report_path[i++]='p'; A_Report_path[i++]='o'; A_Report_path[i++]='r'; 
	 	A_Report_path[i++]='t'; A_Report_path[i++]='.'; A_Report_path[i++]='t'; A_Report_path[i++]='x'; A_Report_path[i++]='t'; 
	 	A_Report_path[i++]=0;

		i=(int)strlen(workspace_name);
	 	ma1_file_path[i++]='_'; ma1_file_path[i++]='W'; ma1_file_path[i++]='O'; ma1_file_path[i++]='R'; ma1_file_path[i++]='K'; 
	 	ma1_file_path[i++]='S'; ma1_file_path[i++]='P'; ma1_file_path[i++]='A'; ma1_file_path[i++]='C'; ma1_file_path[i++]='E'; 
	 	ma1_file_path[i++]='/'; ma1_file_path[i++]='D'; ma1_file_path[i++]='A'; ma1_file_path[i++]='T'; ma1_file_path[i++]='A';
	 	ma1_file_path[i++]='/'; ma1_file_path[i++]='m'; ma1_file_path[i++]='a'; ma1_file_path[i++]='1'; ma1_file_path[i++]='_';
	 	ma1_file_path[i++]='p'; ma1_file_path[i++]='a'; ma1_file_path[i++]='t'; ma1_file_path[i++]='h'; ma1_file_path[i++]='.';
	 	ma1_file_path[i++]='o'; ma1_file_path[i++]='u'; ma1_file_path[i++]='t';
 	
 	}

	if (path_index==supported_functions+1)
 	{ 			 	
	 	for(i=0;i<(int)strlen(workspace_name);i++)
	 	{
			ma1_file_path[i]=workspace_name[i];
			N_Report_path[i]=workspace_name[i];
	  	}		 	
	 	
		i=(int)strlen(workspace_name);
	 	ma1_file_path[i++]='_'; ma1_file_path[i++]='W'; ma1_file_path[i++]='O'; ma1_file_path[i++]='R'; ma1_file_path[i++]='K'; 
	 	ma1_file_path[i++]='S'; ma1_file_path[i++]='P'; ma1_file_path[i++]='A'; ma1_file_path[i++]='C'; ma1_file_path[i++]='E'; 
	 	ma1_file_path[i++]='/'; ma1_file_path[i++]='D'; ma1_file_path[i++]='A'; ma1_file_path[i++]='T'; ma1_file_path[i++]='A';
	 	ma1_file_path[i++]='/'; ma1_file_path[i++]='m'; ma1_file_path[i++]='a'; ma1_file_path[i++]='1'; ma1_file_path[i++]='_';
	 	ma1_file_path[i++]='p'; ma1_file_path[i++]='a'; ma1_file_path[i++]='t'; ma1_file_path[i++]='h'; ma1_file_path[i++]='.';
	 	ma1_file_path[i++]='o'; ma1_file_path[i++]='u'; ma1_file_path[i++]='t';

		i=(int)strlen(workspace_name);
	 	N_Report_path[i++]='_'; N_Report_path[i++]='W'; N_Report_path[i++]='O'; N_Report_path[i++]='R'; N_Report_path[i++]='K'; 
	 	N_Report_path[i++]='S'; N_Report_path[i++]='P'; N_Report_path[i++]='A'; N_Report_path[i++]='C'; N_Report_path[i++]='E'; 
	 	N_Report_path[i++]='/'; N_Report_path[i++]='N'; N_Report_path[i++]='_'; N_Report_path[i++]='r'; N_Report_path[i++]='e';
	 	N_Report_path[i++]='p'; N_Report_path[i++]='o'; N_Report_path[i++]='r'; N_Report_path[i++]='t'; N_Report_path[i++]='.';
	 	N_Report_path[i++]='t'; N_Report_path[i++]='x'; N_Report_path[i++]='t'; N_Report_path[i++]=0; 	
 	} 	
 }

 int getargument(char * argumentline)
 {
	if(strlen(argumentline)!=1)
	{
		return -1;
	}
	else
	{
		return argumentline[0]-48;
	}
 }

 int getargumentSCORE(char * argumentline)
 {
	int i=0,index;
	int value=0;
	int dig;
	int error=-1;
	char argumentlinel[200];
	float expval=0.0;

    for(i=1;i<strlen(argumentline);i++)
	{

		if((argumentline[0]==43 || argumentline[0]==45 || (argumentline[0]>=48 && argumentline[0]<=57)) && (argumentline[i]>=48 && argumentline[i]<=57))
		{
			error=0;
		}
		else
		{
			error=1;
			break;
		}	
	}
	

	if(error==1)
	{
		printf("\n               %s is not valid input!",argumentline);
		printf("\n\n               Enter value:");
		scanf("%s",argumentlinel);
		getargumentSCORE(argumentlinel);

	}
	else
	{

		if(argumentline[0]==43 || argumentline[0]==45) // sign
		{
			index=1;			
		}
		else // number
		{
			index=0;
		}

		expval=0.0;
		value = 0;
		for(i=strlen(argumentline)-1;i>index-1;i--)
		{			
			value=	value+ (argumentline[i]-48)*powf(10.0,expval);
			expval++;
		}
			
		if(index==1)
		{
			if(argumentline[0]==45)
			{
				value = 0 - value; 
			}
		}

		if(value<-100 || value >100)
		{
			printf("\n               %s is not valid input!",argumentline);
			printf("\n\n               Enter value:");
			scanf("%s",argumentlinel);
			getargumentSCORE(argumentlinel);
		}
		else
		{			
			return value;
		}		
	}
 }

 int option_menu()
 {	
	
	int mode=-1;
	char argumentline[150];

	printf("\n\n    1: Consensus Sequence Generation - CSG");
 	printf("\n\n    2: Preliminary File Generation - PFG"); 	
 	printf("\n\n    3: Ambiguous Character Detection - ACD");
 	printf("\n\n    4: Polymorphic Site Detection - PSD");
	printf("\n\n    5: Online Help");
	
 	printf("\n\n\n   Select function:");

	scanf("%s",argumentline);

	mode = getargument(argumentline);


	while(mode!=1 && mode!=2 && mode!=3 && mode!=4 && mode!=5)
	{
		printf("\n\n\n   => Invalid function number! Valid codes are: 1, 2, 3, 4, 5\n");

		printf("\n   Select function:");
 		scanf("%s",argumentline);

		mode = getargument(argumentline);	
	}

	return mode;
 } 

 int option_menu_red()
 {	
	
	int mode=-1;
	char argumentline[150];

	printf("\n\n    0: General Information");
	printf("\n\n    1: Consensus Sequence Generation - CSG");
 	printf("\n\n    2: Preliminary File Generation - PFG"); 	
 	printf("\n\n    3: Ambiguous Character Detection - ACD");
 	printf("\n\n    4: Polymorphic Site Detection - PSD");	
	
 	printf("\n\n\n   Select help section:");

	scanf("%s",argumentline);

	mode = getargument(argumentline);


	while(mode!=0 && mode!=1 && mode!=2 && mode!=3 && mode!=4)
	{
		printf("\n\n\n   > Invalid section number! Valid codes are: 0, 1, 2, 3, 4\n");

		printf("\n   Select section:");
 		scanf("%s",argumentline);

		mode = getargument(argumentline);	
	}

	return mode;
 } 

  void print_help()
 {
	 char percent = '%';

	 if(ONLINE_HELP_ARG==0)
	 {
		 printf("\n\n    General Information\n");
		 printf("    -------------------\n\n");

		 printf("      ChromatoGate is an open-source tool that helps in the detection of\n\n");
		 printf("    alignment  errors  that  have  been  introduced  by  the  sequencer.\n\n");
		 printf("    Through a four-step framework that begins with the  gathering of the\n\n");
		 printf("    under-study sequences into an unaligned fasta file and ends with the\n\n");
		 printf("    generation of  reports that  contain the  chromatogram positions  of\n\n");
		 printf("    those nucleotides  that are responsible for characterizing  the site\n\n");
		 printf("    they belong to  as polymorphic,  the tool  allows the user to easily\n\n");
		 printf("    correct possible errors in the alignment and eventually  improve the\n\n");
		 printf("    quality of the inferred phylogenetic tree.\n\n\n");
		 printf("    The program offers the following functions: \n\n");
		 printf("       1. Consensus Sequence Generation (CSG)\n\n");
		 printf("       2. Preliminary File Generation (PFG)\n\n");
		 printf("       3. Ambiguous Character Detection (ACD)\n\n");
		 printf("       4. Polymorphic Site Detection (PSD)\n\n");
		 printf("    Refer to the respective help subsection  for a detailed description\n\n");
		 printf("    of each function.\n\n\n");
		 printf("    CREATE/SELECT WORKSPACE\n\n");
		 printf("    Initially,  a  workspace is created (or selected if already exists).\n\n");
		 printf("    If a pre-existing  workspace is  selected with the  aim to execute a\n\n"); 
	         printf("    function  that has  already been  executed for  this  workspace in a\n\n");
		 printf("    previous run of the program,  it is advisable the user to remove all\n\n");
		 printf("    files that exist in the workspace from the previous run.\n\n\n");
		 printf("    FILE FORMAT\n\n");
		 printf("    All input sequence files must be in FASTA format.  In addition,  the\n\n");
		 printf("    sequence files must have the extension \".fas\"\n\n\n");
		 printf("    EDGE TRIMMING\n\n");
		 printf("    \"Trim-indicators\"  are used for the trimming of the input  sequences\n\n");
		 printf("    during  the initial gathering of the input  sequences into the FASTA\n\n");
		 printf("    file.\n\n");
		 printf("    A \"trim-indicator\"  is a sequence  of gaps  (typically two or more).\n\n");
		 printf("    The  sequence  of  gaps  can be  manually inserted  into  the  input\n\n");
		 printf("    sequence by replacing an equal number  of nucleotides that belong to\n\n");
		 printf("    the edge to be trimmed.\n\n");
		 printf("    Note that, total absence of  \"trim-indicators\"  still yields a valid\n\n");
		 printf("    input sequence thus allowing for using the sequence exactly as it is\n\n");
		 printf("    when it comes out of the sequencer.\n\n");
		 printf("    Example of the initial sequence: >seq_name  ACGTCGCTCGATATCGATCGCTAG\n\n");	 
		 printf("    Sequence with \"trim-indicators\": >seq_name  ACG---CTCGATATCGATCG--AG\n\n");	 
		 printf("    Sequence with the edges trimmed: >seq_name        CTCGATATCGATCG\n\n\n\n");
		
		 printf("    For additional information email the author: n.alachiotis@gmail.com\n\n\n");
	 }

	 if(ONLINE_HELP_ARG==1)
	 {
		 printf("\n\n    Consensus Sequence Generation - CSG\n");
		 printf("    -----------------------------------\n\n");
		 printf("      The  CSG  function is used to generate  consensus  sequences.  The\n\n");
		 printf("    user must provide as input two sequences, one amplified by a forward\n\n");
		 printf("    primer  and  one amplified by  a reversed primer. In the background,\n\n");
		 printf("    information  regarding  the  origin  of  every  nucleotide  in  each\n\n");
		 printf("    consensus sequence is stored in order to be  used by the ACD and PSD\n\n");
		 printf("    functions.\n\n\n");
		 printf("    1.1 PROMPT\n\n");
		 printf("    Initially,  the tool prompts  the user to place  the input  sequence\n\n");
		 printf("    files  into  the   CONSENSUS\\FORWARD  and  CONSENSUS\\REVERSE  folder\n\n");
		 printf("    according  to  the  primer  each sequence  has been  amplified with.\n\n");
		 printf("    Input sequences must comply with the supported sequence and filename\n\n");
		 printf("    formats: sequences in FASTA format, filenames with extension \".fas\".\n\n");
		 printf("    Trim-indicators  (few-gap sequences)  can appear in the input files.\n\n");
		 printf("    Sequences in the  CONSENSUS\\REVERSE  folder -MUST NOT- represent the\n\n");
		 printf("    reversed complement of the original, sequencer-generated, sequences.\n\n"); 
		 printf("    Apart from the sequence and filename requirements, an additional CSG\n\n");
		 printf("    filename requirement exists:  For  every  file in  the  CSG  FORWARD\n\n");
		 printf("    folder  (e.g., example_seq.fas)  the file in the REVERSE folder that\n\n");
		 printf("    will be used in the generation of the consensus sequence must have\n\n");
		 printf("    exactly the same name: example_seq.fas.\n\n\n");
		 printf("    1.2 PROMPT\n\n");
		 printf("    The  Smith-Waterman  algorithm is used for the alignment  of the two\n\n");
		 printf("    input sequences for every consensus sequence generation.  The second\n\n");
		 printf("    prompt  allows  the user to  select  between  the default  alignment\n\n");
		 printf("    scoring  function  or insert manually  user-defined integers for the\n\n");
		 printf("    match score  and  mismatch and gap penalties. The respective default\n\n");
		 printf("    values have been decided through experiments on real-world sequences\n\n");
		 printf("    and are as follows: MATCH=3, MISMATCH=-3, GAP=-3\n\n\n");
		 printf("    1.3 PROMPT\n\n");
		 printf("    Through the third prompt  the user can  select a  mismatch  handling\n\n");
		 printf("    strategy  (MHS).  The  MHS decides  which ambiguous  character  will\n\n");
		 printf("    appear in the  final consensus  sequence at  the positions  that the\n\n");
		 printf("    alignment algorithm has detected a mismatch.  The AMB MHS places the\n\n");
		 printf("    closest ambiguous character while the N MHS uses the N character for\n\n");
		 printf("    all mismatches.\n\n\n\n");
		 //printf("    1.4 PROMPT\n\n");
		 //printf("    The last CSG prompt allows the inspection of the alignment  process.\n\n");
		 //printf("    If inspection is selected,  an additional folder (CONSENSUS\\INSPECT)\n\n");
		 //printf("    is generated and  a three-sequence  alignment is  created for  every\n\n");
		 //printf("    consensus sequence.  The three sequences are the aligned  consensus,\n\n");
		 //printf("    forward and reversed sequences.\n\n\n\n");
		 
		 printf("    For additional information email the author: n.alachiotis@gmail.com\n\n\n");
	 }

	 if(ONLINE_HELP_ARG==2)
	 {
		 printf("\n\n    Preliminary File Generation - PFG\n");
		 printf("    ---------------------------------\n\n");
		 printf("      The PFG function can be used to generate a preliminary FASTA file.\n\n");
		 printf("    The sequences in this file are  not aligned. The user can calculate\n\n");
		 printf("    a multiple alignment of these sequences using any alignment program.\n\n");
		 printf("    In the background,  information regarding  the origin of every base\n\n");
		 printf("    in each input sequence is stored in order to be used by the ACD and\n\n");
		 printf("    PSD functions.\n\n\n");
		 printf("    PROMPT\n\n");
		 printf("    The tool prompts the user to place the input sequence files into the\n\n");
		 printf("    FORWARD and REVERSE folders of the selected  workspace according  to\n\n");
		 printf("    the  primer  each sequence  has been  amplified with.\n\n");
		 printf("    Input sequences must comply with the supported sequence and filename\n\n");
		 printf("    formats: sequences in FASTA format, filenames with extension \".fas\".\n\n");
		 printf("    Trim-indicators  (few-gap sequences)  can appear in the input files.\n\n");
		 printf("    Sequences in the  REVERSE  folder -MUST NOT- represent the  reversed\n\n");
		 printf("    complement of the original, sequencer-generated, sequences.\n\n"); 
		 printf("    Consensus sequences that have been generated using the CSG  function\n\n");
		 printf("    and thus already exist in the  CONSENSUS folder are included in  the\n\n");
		 printf("    preliminary file automatically.\n\n\n\n");
	
		 printf("    For additional information email the author: n.alachiotis@gmail.com\n\n\n");
	 }

	 if(ONLINE_HELP_ARG==3)
	 {
		 printf("\n\n    Ambiguous Character Detection - ACD\n");
		 printf("    -----------------------------------\n\n");
		 printf("      The ACD function can be used to target the ambiguous characters in\n\n");
		 printf("    an input multiple alignment file that  has been calculated from  the\n\n");
		 printf("    preliminary  file generated by the  PFG function.  Using information\n\n");
		 printf("    gathered during the CSG and PFG processess, the ACD function creates\n\n");
		 printf("    reports that  contain the positions of  each ambiguous character  in\n\n");
		 printf("    the input alignment as well as the respective chromatogram positions\n\n");
		 printf("    in order to allow the user to correct possible errors.\n\n\n");
		 printf("    PROMPT\n\n");
		 printf("    The tool prompts the user to place the input alignment file into the\n\n");
		 printf("    workspace directory.\n\n");
		 printf("    Input sequences must comply with the supported sequence and filename\n\n");
		 printf("    formats: sequences in FASTA format,  filename with extension \".fas\".\n\n");
		 printf("    Optionally, the input alignment file can contain sequences that were\n\n");
		 printf("    not present in the PFG-generated preliminary file.  In this case, it\n\n");
		 printf("    is assumed that no chromatograms are available and thus chromatogram\n\n");
		 printf("    positions for nucleotides  that belong  to  these sequences are  not\n\n");
		 printf("    calculated.  If there are chromatograms for  these sequences  repeat\n\n");
		 printf("    the PFG including them.\n\n");
		 printf("    The ACD function  generates an ACD report for every ambiguity in the\n\n");
		 printf("    alignment.  Each report consists of four columns of data denoted as:\n\n");
		 printf("    1. Sequence name\n\n");
		 printf("    2. Alignment site\n\n");
		 printf("    3. Chromatogram position\n\n");
		 printf("    4. Changed to\n\n");
		 printf("    The chromatogram position format is X.Y with X refering to the  type\n\n");
		 printf("    of primer  the respective sequence has been amplified with  while  Y\n\n");
		 printf("    being the chromatogram position.\n\n");
		 printf("    For sequences retrieved from the FORWARD folder X = FW.\n\n");
		 printf("    For sequences retrieved from the REVERSE folder X = RV.\n\n");
		 printf("    For sequences retrieved from the CONSENSUS folder  the  chromatogram\n\n");
		 printf("    position appears as FW.Y - RV.Y.\n\n"); 
		 printf("    The \"Changed to\" field in all reports is formed as: '>?'. The aim of\n\n");
		 printf("    existence of this field is described in the PSD help section.\n\n\n\n");

		 printf("    For additional information email the author: n.alachiotis@gmail.com\n\n\n");
	
	 }

	 if(ONLINE_HELP_ARG==4)
	 {
		 printf("\n\n    Polymorphic Site Detection - PSD\n");
		 printf("    --------------------------------\n\n");
		 printf("      The PSD function is used to detect all the polymorphic sites in an\n\n");
		 printf("    input alignment  based on a  user-defined  criterion.  A PSD  report\n\n");
		 printf("    that contains  chromatogram positions of the nucleotides responsible\n\n");
		 printf("    for polymorphic sites  in  the alignment  is generated in  order  to\n\n");
		 printf("    allow the user to correct possible errors introduced in the sequence\n\n");
		 printf("    by the sequencer. Furthermore, a post-PSD ACD report comprising  all\n\n");
		 printf("    the ambiguous characters in the input alignment is generated.\n\n\n");
		 printf("    4.1 PROMPT\n\n");
		 printf("    The tool prompts the user to place the input alignment file into the\n\n");
		 printf("    workspace directory. This alignment can either be the same one  used\n\n");
		 printf("    as input in the ACD function or  a newly calculated one.  If the ACD\n\n");
		 printf("    input alignment has been  corrected based on the ACD reports,  it is\n\n");
		 printf("    advisable  to  degap  and  realign  the sequences  and  use the  new\n\n");
		 printf("    alignment as input to the PSD function.\n\n");
 		 printf("    Input sequences must comply with the supported sequence and filename\n\n");
		 printf("    formats: sequences in FASTA format,  filename with extension \".fas\".\n\n");
		 printf("    Optionally, the input alignment file can contain sequences that were\n\n");
		 printf("    not present  in the  PFG-generated  preliminary file or in the input\n\n");
		 printf("    alignment file to the ACD function. In this case, it is assumed that\n\n");
		 printf("    no chromatograms are available and thus  chromatogram  positions for\n\n");
		 printf("    bases  that  belong to these  sequences will  not be calculated.  If\n\n");
		 printf("    chromatogram files  for these sequences exist,  repeat  PFG and  ACD\n\n");
		 printf("    functions including them.\n\n\n");
		 printf("    4.2 PROMPT\n\n");
		 printf("    The second prompt allows the user to enter a sensitivity value. This\n\n");
		 printf("    number is used in the selection of the polymorphic sites.  The input\n\n");
		 printf("    can either be  an absolute value,  meaning  that it must be  less or\n\n");
		 printf("    equal  to the number  of sequences  in  the input  alignment, or,  a\n\n");
		 printf("    percentage.  If the input value is a percentage it must  be followed\n\n");
		 printf("    by '%c' otherwise it is treated as an absolute value.\n\n\n",percent);
		 printf("    4.3 PROMPT\n\n");
		 printf("    This prompt allows the user to select search strategy. There are two\n\n");
		 printf("    options:  1. Less or Equal (LE)  and  2. One Major Class (1MC).  The\n\n");
		 printf("    search strategy determines the way  the sensitivity value is used to\n\n");
		 printf("    detect possible polymorphic sites.\n\n");
		 printf("    1. LE:  Selects the sites which contain characters that  appear less\n\n");
		 printf("            or equal times than the sensitivity value.\n\n");
		 printf("            -- Example: A column with  50 As,  50 Cs,  50 Gs and  2 T is\n\n");
		 printf("                        considered  polymorphic  if  the  sensitivity is\n\n");
		 printf("                        equal or greater than 2 (absolute value).\n\n");
		 printf("    1. 1MC:  Selects the sites which contain characters that appear less\n\n");
		 printf("            or equal times than the sensitivity  value and all remaining\n\n");
		 printf("            characters belong to the same nucleotide class.\n\n");
		 printf("            -- Example: A column with  150 As, 1 C and 1 G is considered\n\n");
		 printf("                        polymorphic if the sensitivity is equal to 2.\n\n");
		 printf("    The  main  difference between  the alternative  search strategies is\n\n");
		 printf("    that the 1MC strategy searches for  one major nucleotide  class in a\n\n");
		 printf("    site before applying the  sensitivity-based criterion  while the  LE\n\n");
		 printf("    allows the existence of more than one major classes.\n\n\n");
		 printf("    If errors have been corrected in the sequences by the replacement of\n\n");
		 printf("    ambiguous characters with  nucleotides then the user must update the\n\n");
		 printf("    respective entries  in the ACD  reports by replacing the  symbol '?'\n\n");
		 printf("    in the  \"Changed to\"  field with the correction.  Every  replacement\n\n");
		 printf("    of the '?' symbol in any ACD report must correspond to a replacement\n\n");
		 printf("    of an ambiguous character in the alignment with another base.\n\n");
		 printf("    If one  does  not want  to correct  an ambiguous  character  in  the\n\n");
		 printf("    alignment,  the  respective  ACD report entry  must not be  updated.\n\n");
		 printf("    In that case, the respective '?'  symbol in the  ACD report will  be\n\n");
		 printf("    treated as the pre-existing ambiguous character at the corresponding\n\n");
		 printf("    position in the sequence.\n\n");
		 printf("    - REMEMBER -  When reversed-primer amplified sequences are concerned\n\n");
		 printf("    the '?' symbol  needs to be replaced with the  complementary base of\n\n");
		 printf("    the one that appears in the chromatogram.\n\n\n");
		 printf("    The PSD function generates a PSD report  that contains an entry  for\n\n");
		 printf("    every polymorphic site in the alignment. The report consists of four\n\n");
		 printf("    columns of data denoted as:\n\n");
		 printf("    1. Alignment site\n\n");
		 printf("    2. Site report\n\n");
		 printf("    3. Sequence name\n\n");
		 printf("    4. Chromatogram position\n\n");
		 printf("    The site report provides the number of nucleotides the site consists\n\n");
		 printf("    of.\n\n");
		 printf("    The chromatogram position  format is X.Y.Z with X again refering  to\n\n");
		 printf("    the type of primer the  respective sequence has been amplified with,\n\n");
		 printf("    Y being the chromatogram position and Z being the nucleotide at that\n\n");
		 printf("    position.  For forward-primer  amplified sequences Z represents  the\n\n");
		 printf("    nucleotide in  the sequence as  well as the chromatogram  while  for\n\n");
		 printf("    reversed-primer  amplified sequences  Z  is the  complement  of  the\n\n");
		 printf("    nucleotide in the sequence in order to match with the nucleotide  in\n\n");
		 printf("    the chromatogram.\n\n");
		 printf("    Furthermore,  a post-PSD  ACD  report  containing a list of all  the\n\n");
		 printf("    remaining ambiguous characters in the alignment is generated.\n\n\n\n");

		 printf("    For additional information email the author: n.alachiotis@gmail.com\n\n\n");

	 }
 }
int check_argumentline(char * argumentline)
{

	int i=0;
	int result =0;

	for(i=0;i<strlen(argumentline)+1;i++)
	{
		if(argumentline[i]=='%')
		{
			result=1;
		}
	}
	return result;
}

int getargumentINT(char * argumentline)
{
	int i;
	int factor=1;
	int result=0;
	int value;

	for(i=strlen(argumentline)-1;i>-1;i--)
	{
		value = argumentline[i] - 48;
		if(value<0 || value>9){return -1;}

		result = result + value*factor;

		factor = factor * 10;
	}
	return result;
}


float getargumentFLOAT(char * argumentline)
{
	int i=0;
	int dotpos=strlen(argumentline);
	float result;
	int value;
	int index;

	if(argumentline[strlen(argumentline)-1]=='%')
	{
    	argumentline[strlen(argumentline)-1]=0;
	}

	for(i=0;i<strlen(argumentline);i++)
	{
		if(argumentline[i]!='.')
		{
			value = argumentline[i] - 48;
			if(value<0 || value>9){	return -1.0;}
		}
		else
		{
			dotpos=i;
		}
	}
	argumentline[dotpos] = 0;

	return (float)getargumentINT(argumentline);
	
}

 int count_sequences (FILE * fp)
 {

	char temp;
	int counter=0;
	temp = fgetc(fp);

	while(temp!=EOF)
	{
		if(temp=='>')
		{
			counter++;
		}
		temp = fgetc(fp);
	}

	return counter;
 }

 float set_upperlimit(int is_percentage, int sequences_num)
 {
	 float upperlimit=0;

	 if(is_percentage==0)
	 {
		upperlimit=(float)sequences_num;
	 }
	 else
	 {
		upperlimit=100.0;
	 }
	 return upperlimit;
 }


 int print_menu()
 {
 	int i=0;
 	int mode;
	int stop=-1;
	char percent = '%';
	char wait;
 	
 	char workspace_name[100];
 	char workspace[110];
 	char tmp[100];
	char done;
	int program_check=-1;
	int auto_cor_run_arg=-1;
	char argumentline[200];
	int sequences_num=0;
	float upperlimit=0;

 	
	FILE *fp;
 	
	printf("\n\n\n  CREATE/SELECT WORKSPACE\n  -----------------------");
	printf("\n\n   Workspace name:");
	
 	scanf("%s",workspace_name);

	printf("\n\n  SELECT FUNCTION\n  ---------------");

 	create_workspace(workspace_name);
 	
	stop=0;;
	while(stop==0)
	{
 		mode = option_menu();

		if(mode==CONSENSUS_RUN || mode==PRE_ALIGNMENT_RUN || mode==AMBIGUITIES_RUN || mode==DIVERSITY_RUN || mode==ONLINE_HELP)
		{	
			stop=1;
		}
		else
		{
			stop=0;
		}
	}
 	

	
	if(mode>0 && mode<supported_functions+1)
	{
		if(mode==CONSENSUS_RUN)
 		{
			
			create_consensus_folders(workspace_name);
 		
			printf("\n\n  1. CONSENSUS SEQUENCE GENERATION\n  --------------------------------");
			printf("\n\n   1.1: Place the sequence files into the ..\\CONSENSUS\\FORWARD and\n\n        ..\\CONSENSUS\\REVERSE folders and press Enter.\n");
 			
			scanf("%c",&wait);scanf("%c",&wait);

			CONSENSUS_CALC_INSPECTION = -1;
			DEFAULT_SCORING_SCHEME = -1;
			CLOSEST_AMBIGUITY_REPLACEMENT =-1;
 				
 		}
 		if(mode==PRE_ALIGNMENT_RUN)
 		{
 		
			printf("\n\n  2. PRELIMINARY FILE GENERATION\n  ------------------------------");
			printf("\n\n   Place the sequence files into the FORWARD and REVERSE folders and\n\n   press Enter.\n");
 			
 			printf("\n   Consensus files (CONSENSUS folder), if any, will be included automatically.\n");

			scanf("%c",&wait);scanf("%c",&wait);
			
 		}
 		if(mode==AMBIGUITIES_RUN)
 		{
			printf("\n\n  3. AMBIGUOUS CHARACTER DETECTION\n  --------------------------------");

			fp = fopen (lut_out_path,"r");
	 		if(fp==NULL) 
			{ 
				printf("\n\n\n\n   => ../%s file is missing.\n\n",lut_out_path);
				printf("      Execute function (%d) in order to resolve this issue.\n\n",PRE_ALIGNMENT_RUN);

				printf("\n  Press Enter to exit...\n\n");
				scanf("%c",&done);scanf("%c",&done);

				exit(0); 
			}
			fclose(fp);
			fp = fopen (pre_alignment_out_path,"r");
			if(fp==NULL) 
			{ 
				printf("\n\n\n\n   => ../%s file is missing.\n\n",pre_alignment_out_path);
				printf("      Execute function (%d) in order to resolve this issue.\n\n",PRE_ALIGNMENT_RUN);

				printf("\n  Press Enter to exit...\n\n");
				scanf("%c",&done);scanf("%c",&done);

				exit(0); 
			}
			fclose(fp);

			printf("\n\n   Place the alignment file into the ../%s_WORKSPACE/ folder.",workspace_name);
 			
			printf("\n\n   Enter the alignment filename:");
 	 		scanf("%s",tmp);
 	
			run_arg=AMBIGUITIES_RUN;
 			create_path(run_arg,tmp,workspace_name);

			if(ma1_name[strlen(ma1_name)-4]!='.')
			{
				ma1_name[strlen(ma1_name)]='.';
				ma1_name[strlen(ma1_name)]='f';
				ma1_name[strlen(ma1_name)]='a';
				ma1_name[strlen(ma1_name)]='s';
				ma1_name[strlen(ma1_name)]=0;

			}
			fp = fopen (ma1_name,"r");
			
			if(fp==NULL) 
			{ 
				printf("\n\n\n\n   => ../%s file is missing.\n\n",ma1_name);
				
				printf("\n  Press Enter to exit...\n\n");
				scanf("%c",&done);scanf("%c",&done);

				exit(0); 
			}
			fclose(fp);
 		
 			fp = fopen(ma1_file_path,"w");
 			fprintf(fp,"%s",ma1_name);
 			fclose(fp);
 	
 		}
 		if(mode==DIVERSITY_RUN)
 		{
 			printf("\n\n  4. POLYMORPHIC SITE DETECTION\n  -----------------------------");

			fp = fopen (lut_out_path,"r");
	 		if(fp==NULL) 
			{ 
				printf("\n\n\n\n   => ../%s file is missing.\n\n",lut_out_path);
				printf("      Execute function (%d) in order to resolve this issue.\n\n",PRE_ALIGNMENT_RUN);

				printf("\n  Press Enter to exit...\n\n");
				scanf("%c",&done);scanf("%c",&done);

				exit(0); 
			}
			fclose(fp);
			fp = fopen (pre_alignment_out_path,"r");
			if(fp==NULL) 
			{ 
				printf("\n\n\n\n   => ../%s file is missing.\n\n",pre_alignment_out_path);
				printf("      Execute function (%d) in order to resolve this issue.\n\n",PRE_ALIGNMENT_RUN);

				printf("\n  Press Enter to exit...\n\n");
				scanf("%c",&done);scanf("%c",&done);

				exit(0); 
			}
			fclose(fp);

			
			create_path(supported_functions+1,NULL,workspace_name);

			fp = fopen(ma1_file_path,"r");
			if(fp==NULL) 
			{ 
				printf("\n\n\n\n   => ../%s file is missing.\n\n",ma1_file_path);
				printf("      Execute function (%d) in order to resolve this issue.\n\n",AMBIGUITIES_RUN);

				printf("\n  Press Enter to exit...\n\n");
				scanf("%c",&done);scanf("%c",&done);

				exit(0); 
			}
 			fscanf(fp,"%s",ma1_name); 	    
 			fclose(fp);

			/*
			fp = fopen (N_Report_path,"r");
			if(fp==NULL) 
			{ 
				//printf("\n\n\n\n   => ../%s file is missing.\n\n",N_Report_path);
				//printf("      Execute function (%d) in order to resolve this issue.\n\n",AMBIGUITIES_RUN);

				//printf("\n  Press Enter to exit...\n\n");
				//scanf("%c",&done);scanf("%c",&done);

				//exit(0); 
			}
			else
			{
 			fclose(fp);
			}
			*/

			printf("\n\n   4.1: Place the alignment file into the ../%s_WORKSPACE/ folder.",workspace_name);

 			printf("\n\n        Enter the alignment filename:");
 			scanf("%s",tmp);

 			run_arg=DIVERSITY_RUN;
 			
			create_path(run_arg,tmp,workspace_name);

			if(ma2_name[strlen(ma1_name)-4]!='.')
			{
				ma2_name[strlen(ma2_name)]='.';
				ma2_name[strlen(ma2_name)]='f';
				ma2_name[strlen(ma2_name)]='a';
				ma2_name[strlen(ma2_name)]='s';
				ma2_name[strlen(ma2_name)]=0;

			}
			fp = fopen (ma1_name,"r");

			fp = fopen (ma2_name,"r");
			if(fp==NULL)
			{ 
				printf("\n\n\n\n   => ../%s file is missing.\n\n",ma2_name);				

				printf("\n  Press Enter to exit...\n\n");
				scanf("%c",&done);scanf("%c",&done);

				exit(0); 
			}
			sequences_num = count_sequences(fp);

			printf("\n        The input alignment file contains %d sequences!\n",sequences_num);

			fclose(fp);

			
			printf("\n\n   4.2: Select PSD sensitivity.");
			printf("\n\n        Include '%c' if you are giving percentage!",percent);
			printf("\n\n        Sensitivity:");
			
			scanf("%s",argumentline);

			is_percentage = check_argumentline(argumentline);

			diversity_thres_perc = getargumentFLOAT(argumentline);

			upperlimit = set_upperlimit(is_percentage,sequences_num);

			while(diversity_thres_perc>upperlimit || diversity_thres_perc<0.0)
			{
					printf("\n\n        %s is not valid input!\n",argumentline);

					printf("\n        Valid input range: 0 - %d or 0.0%c - 100.0%c",sequences_num, percent, percent);
					
					printf("\n\n        Sensitivity:");
 				
					scanf("%s",argumentline);

					is_percentage = check_argumentline(argumentline);
					
					diversity_thres_perc = getargumentFLOAT(argumentline);
			
					upperlimit = set_upperlimit(is_percentage,sequences_num);
			}


			printf("\n\n   4.3: Select PSD search strategy.");
			printf("\n\n        1. LE  (Less or Equal)");
			printf("\n\n        2. 1MC (One Major Class)");
			printf("\n\n        Select strategy:");

			scanf("%s",argumentline);

			PSDsearch = getargumentINT(argumentline);

		
			while(PSDsearch!=1 && PSDsearch !=2)
			{
					printf("\n\n        %s is not valid input!\n",argumentline);
					printf("\n\n        1. LE  (Less or Equal)");
					printf("\n\n        2. 1MC (One Major Class)");
				    printf("\n\n        Select strategy:");
 				
					scanf("%s",argumentline);

					PSDsearch = getargumentINT(argumentline);
			}




			/*}
			else
			{
				while((diversity_thres_perc>100.0 || diversity_thres_perc<0.0) || (is_percentage==0))
				{
					printf("\n\n        %s is not valid input!\n",argumentline);
					  printf("\n       Valid input range: 0.0%c - 100.0%c",percent,percent);

					printf("\n\n        Sensitivity:",percent);
 				
					scanf("%s",argumentline);		

					is_percentage = check_argumentline(argumentline);

					diversity_thres_perc = getargumentFLOAT(argumentline);
				}
			}*/

			/*printf("\n\n   4.3: Are the ACD report(s) updated?");
			printf("\n\n        1: YES \n\n        2: NO");
			printf("\n\n        Select:");
			scanf("%s",argumentline);

			AMBIGUITY_ERROR_CHECK = getargument(argumentline);
			
	
			while(AMBIGUITY_ERROR_CHECK!=1 && AMBIGUITY_ERROR_CHECK!=2)
			{
				
				printf("\n\n        %s is not valid input!",argumentline);
				printf("\n\n        1: YES \n\n        2: NO");
				printf("\n\n        Select:");
						
				scanf("%s",argumentline);

				AMBIGUITY_ERROR_CHECK = getargument(argumentline);

			}
			*/
 		}
		if(mode==ONLINE_HELP)
 		{
			printf("\n\n  5. CHROMATOGATE ONLINE HELP\n  ---------------------------");

			ONLINE_HELP_ARG = option_menu_red();

			print_help();

			printf("\n    Press Enter to exit...\n\n");
			scanf("%c",&done);scanf("%c",&done);
		}
	}
	else
	{		
			printf("\n      Press Enter to exit...\n\n");
			scanf("%c",&done);scanf("%c",&done);
	}

 	return mode;

 }

 void adjust_threshold(int sequence_number)
 {
	 float tmp, fval;
	 int val;
	
	 if(diversity_thres_perc==0)
	 {
		 diversity_thres=0;
	 }
	 else
	 {
	
		 tmp = (float)sequence_number;
		 tmp = tmp * diversity_thres_perc ;
		 tmp = tmp /100.0;

		 val=floor(tmp);
	 
		 fval = (float)val;

		 if(tmp - fval < 0.5)
		 {
			 diversity_thres = val;
		 }
		 else
		 {
			 diversity_thres = val + 1;
		 }

		 if(diversity_thres==0)
		 {
			 diversity_thres++;
		 }
	 }

	 if(is_percentage==0)
	 {
		 diversity_thres = diversity_thres_perc;
	 }
 }

 int match_forward_to_reverse_all(int index)
 {
	 int i;
	 int found=0;
	 
	 for(i=0;i<consensus_reverse_list.size;i++)
	 {
		if(strcmp(consensus_forward_list.data[index],consensus_reverse_list.data[i])==0)
 		{
 			found++;
			break;
 		}
	 }

	 if(found==0)
	 {
		printf("\n\n\n\n   => File \"%s\" is missing from folder\n\n      \"../%s\"!\n\n",consensus_forward_list.data[index],Consensus_Reverse_Path);
	 }

	return found;
	 
 }

 int match_reverse_to_forward_all(int index)
 {
	 int i;
	 int found=0;
	 
	 for(i=0;i<consensus_forward_list.size;i++)
	 {
		if(strcmp(consensus_reverse_list.data[index],consensus_forward_list.data[i])==0)
 		{
 			found++;
			break;
 		}
	 }

	 if(found==0)
	 {
		printf("\n\n\n\n   => File \"%s\" is missing from folder\n\n      \"../%s\"!\n\n",consensus_reverse_list.data[index],Consensus_Forward_Path);
	 }
	
	 return found;
 }

 consensus_match_struct * match_forward_to_reverse(int index)
 {
	 int i;
	 consensus_match_struct * result;
	 result = (consensus_match_struct *)(malloc(sizeof(consensus_match_struct)));

	 result->forward_index=index;

	 for(i=0;i<consensus_forward_list.size;i++)
	 {
		if(strcmp(consensus_forward_list.data[index],consensus_forward_list.data[i])==0)
 		{
			result->reverse_index=i;
 			break;
 		}
	 }

	 return result;

 }

 void create_file_path(int fw_rv_sel,int index)
 {
	 int i,j;

	 for(i=0;i<strlen(Consensus_Forward_Path);i++)
	 {
		 if(fw_rv_sel==0)
		 {
			filepath[i]=Consensus_Forward_Path[i];
		 }
		 else
		 {
			 filepath[i]=Consensus_Reverse_Path[i];
		 }

		 filepath[i+1]='/';
	 }

	 i++;

	 for(j=i;j<i+strlen(consensus_forward_list.data[index]);j++)
	 {
		 filepath[j]=consensus_forward_list.data[index][j-i];
		 filepath[j+1]=0;
	 }

	

 }

 void consensus_input_data_check()
 {
	 char done;
	 int i;
	 int proceed=-1;

	 consensus_match_struct * consensus_match;

	 if(consensus_forward_list.size+consensus_reverse_list.size==0)
	 {		 
		 printf("\n\n\n\n   => Folders ../%s and\n\n\t      ../%s are empty!\n\n",Consensus_Forward_Path,Consensus_Reverse_Path);
		 
		 printf("\n  Press Enter to exit...\n\n");
		 scanf("%c",&done);

		 exit(0);
	 }
	 else
	 {
		 if(consensus_forward_list.size>consensus_reverse_list.size)
		 {
			for(i=0;i<consensus_forward_list.size;i++)
			{
				 proceed=match_forward_to_reverse_all(i);
				 
				 if(proceed==0)
				 {
					printf("\n  Press Enter to exit...\n\n");
					scanf("%c",&done);

					exit(0);
				 }

			}
		 }
		 else
		 {
			for(i=0;i<consensus_reverse_list.size;i++)
			{
				 proceed=match_reverse_to_forward_all(i);

				 if(proceed==0)
				 {
					printf("\n  Press Enter to exit...\n\n");
					scanf("%c",&done);

					exit(0);
				 }
			}
		 }
	 }

 }

 void get_filename_from_filepath(char * filepath,char * sequence_name)
 {
	 int i,j;
	 int filepath_len = strlen(filepath);
	 int sequence_name_len = 0;

	 for(i=filepath_len;i>-1;i--)
	 {
		 sequence_name_len++;
		 if(filepath[i]=='/')
		 {
			 break;
		 }
	 }
	 sequence_name[0]='>';
	 for(i=1;i<sequence_name_len;i++)
	 {
		sequence_name[i]=filepath[filepath_len-sequence_name_len+i+1];
		if(sequence_name[i]=='.')
		{
			sequence_name[i]=0;
			break;
		}
	 }	 
 }

 seq_f_cons * get_seq_F_consensus(int index)
 {
	char t_ch,done;
	int c_ind=0;
	int i=0;
	int charind=0;
	seq_f_cons * temp;
	FILE * fp;
	char trash[500];

	temp = (seq_f_cons *)malloc(sizeof(seq_f_cons));
	
	temp->seq_fw_uncut = (char *) malloc(sizeof(char)*1600);
	temp->seq_rv_uncut = (char *) malloc(sizeof(char)*1600);

	create_file_path(0,index);

	fp=fopen(filepath,"r");

	if(fp==NULL)
	{
		printf("\n\n\n\n   => ../%s file can not be read by ChromatoGate.\n\n",filepath);				

		printf("\n  Press Enter to exit...\n\n");
		scanf("%c",&done);

		exit(0);
	}

	fscanf(fp,"%s",temp->seq_name);

	get_filename_from_filepath(filepath,temp->seq_name);


	t_ch = fgetc(fp);

	t_ch = fgetc(fp);
	charind++;
	while(t_ch!=EOF)
	{
		if (t_ch=='A' || t_ch=='C' || t_ch=='G' || t_ch=='T' || t_ch=='-' || t_ch=='N' ||
			t_ch=='U' || t_ch=='M' || t_ch=='R' || t_ch=='W' || t_ch=='S' || t_ch=='Y' ||
			t_ch=='K' || t_ch=='V' || t_ch=='H' || t_ch=='D' || t_ch=='B' ) 
		{
			temp->seq_fw_uncut[c_ind++]=t_ch;
			temp->seq_fw_uncut[c_ind+1]=0;
		}
		else
		{
			if((t_ch!='\n') && (t_ch>=32 && t_ch<=126))
			{
				
				printf("\n\n\n\n   => Invalid character (%c-%d) in file: %s.\n\n",t_ch,charind,filepath);				

				printf("\n  Press Enter to exit...\n\n");
				scanf("%c",&done);

				exit(0); 

			}

		}
		t_ch = fgetc(fp);
	}
	
	temp->len_fw_uncut = c_ind;

	fclose(fp);


	c_ind=0;
	create_file_path(1,index);

	fp=fopen(filepath,"r");

	if(fp==NULL)
	{
		printf("\n\n\n\n   => ../%s file can not be read by ChromatoGate.\n\n",filepath);				

		printf("\n  Press Enter to exit...\n\n");
		scanf("%c",&done);

		exit(0);
	}

	fscanf(fp,"%s",trash);

	t_ch = fgetc(fp);

	t_ch = fgetc(fp);
	while(t_ch!=EOF)
	{
		if (t_ch=='A' || t_ch=='C' || t_ch=='G' || t_ch=='T' || t_ch=='-' || t_ch=='N' ||
			t_ch=='U' || t_ch=='M' || t_ch=='R' || t_ch=='W' || t_ch=='S' || t_ch=='Y' ||
			t_ch=='K' || t_ch=='V' || t_ch=='H' || t_ch=='D' || t_ch=='B' ) 
		{
			temp->seq_rv_uncut[c_ind++]=t_ch;
			temp->seq_rv_uncut[c_ind+1]=0;
		}
		t_ch = fgetc(fp);
	}
	
	temp->len_rv_uncut = c_ind;

	fclose(fp);

	return temp;
 }


 void read_valid_consensus_data()
 {
	 int i;
	 FILE *fp;
	 char done;

	 cons_list = (seq_f_cons **)(malloc(sizeof(seq_f_cons*)*consensus_forward_list.size));

	 for(i=0;i<consensus_forward_list.size;i++)
	 {
		 cons_list[i]=get_seq_F_consensus(i);		 
	 }

 }

 void read_data_consensus()
 {

	int type = 0;
	int i=0,j=0;
	FILE * fpF ;
	FILE * fpR ;
	TCHAR tempdirect[200];
	char workspacedirectory[200];
	char foldern[8];
	char foldern_pre[10];
	char done;	
	int table_index=0;
	

	for(i=0;i<200;i++)
	{
		workspacedirectory[i]=Consensus_Forward_Path[i];

		if(workspacedirectory[i]=='/')
		{
			workspacedirectory[i]=0;
			break;
		}
	}
	
	i=0;
	foldern[i++]='F';foldern[i++]='O';foldern[i++]='R';foldern[i++]='W';
	foldern[i++]='A';foldern[i++]='R';foldern[i++]='D';foldern[i++]=0;
 	
	consensus_forward_list.size = read_all_X_files_consensus(0,0,0, workspacedirectory, foldern);

	consensus_forward_list.data = (char**)(malloc(sizeof(char*)*consensus_forward_list.size));

	for(i=0;i<consensus_forward_list.size;i++)
	{
		consensus_forward_list.data[i] = (char*)(malloc(sizeof(char)*100));

		for(j=0;j<100;j++)
		{
			consensus_forward_list.data[i][j]=0;
		}

	}
	cons_fw_index=0;

	read_all_X_files_consensus(1,0,0, workspacedirectory, foldern);
	
	i=0;
	foldern[i++]='R';foldern[i++]='E';foldern[i++]='V';foldern[i++]='E';
	foldern[i++]='R';foldern[i++]='S';foldern[i++]='E';foldern[i++]=0;
	
	consensus_reverse_list.size = read_all_X_files_consensus(0,0,1, workspacedirectory, foldern);

	consensus_reverse_list.data = (char**)(malloc(sizeof(char*)*consensus_reverse_list.size));

	for(i=0;i<consensus_reverse_list.size;i++)
	{
		consensus_reverse_list.data[i] = (char*)(malloc(sizeof(char)*100));

		for(j=0;j<100;j++)
		{
			consensus_reverse_list.data[i][j]=0;
		}
	}
	cons_rv_index=0;

	read_all_X_files_consensus(1,0,1, workspacedirectory, foldern);

	consensus_input_data_check();	
	
	read_valid_consensus_data();

 }



int SW_MATCH = 0;
int SW_MISMATCH = 0;
int SW_INDEL = 0;

int SW_MATCH_DEF = 3;
int SW_MISMATCH_DEF = -3;
int SW_INDEL_DEF = -3;

int SW_MATCH_USR = 0;
int SW_MISMATCH_USR = 0;
int SW_INDEL_USR = 0;

int max_2 (int arg1, int arg2)
{
	if(arg1>=arg2)
	{
		return arg1;
	}
	else
	{
		return arg2;
	}
}

int max_4 (int arg1, int arg2, int arg3, int arg4)
{
	int arg1_2 , arg3_4, arg1_2_3_4;
	
	arg1_2 = max_2 (arg1, arg2);
	arg3_4 = max_2 (arg3, arg4);

	arg1_2_3_4 = max_2(arg1_2,arg3_4);

	return arg1_2_3_4;
}

int max_2_mv (int arg1, int arg2)
{
	if(arg1>=arg2)
	{
		return 1;
	}
	else
	{
		return 2;
	}
}

int max_3_mv (int arg1, int arg2, int arg3)
{
	int arg1_2_sel, arg1_2, arg1_2_3_sel;
	
	arg1_2_sel = max_2_mv (arg1, arg2);

	if(arg1_2_sel==1)
	{
		arg1_2 = arg1;
	}
	else
	{
		arg1_2 = arg2;
	}


	arg1_2_3_sel = max_2_mv (arg1_2, arg3);

	if(arg1_2_3_sel==1)
	{
		if(arg1_2_sel==1)
		{
			return 1;
		}
		else
		{
			return 2;
		}
	}
	else
	{
		return 3;
	}
}

int char_to_code(char input)
{
	int code=0;

	switch(input)
	{
		case 'A':
			code=1;
			break;
		case 'C':
			code=2;
			break;
		case 'G':
			code=4;
			break;
		case 'T':
			code=8;
			break;
		case 'N':
			code=15;
			break;
		case 'M':
			code=3;
			break;
		case 'R':
			code=5;
			break;
		case 'W':
			code=9;
			break;
		case 'S':
			code=6;


			break;
		case 'Y':
			code=10;
			break;
		case 'K':
			code=12;
			break;
		case 'V':
			code=7;
			break;
		case 'H':
			code=11;
			break;
		case 'D':
			code=13;
			break;
		case 'B':
			code=14;
			break;
		default:
			code=0;
			break;

	}

	return code;
}

char code_to_char(int input)
{
	char char_val;

	if(CLOSEST_AMBIGUITY_REPLACEMENT==1)
	{
		switch(input)
		{
			case 1:
				char_val = 'A';
				break;
			case 2:
				char_val = 'C';
				break;
			case 4:
				char_val = 'G';
				break;
			case 8:
				char_val = 'T';
				break;
			case 15:
				char_val = 'N';
				break;
			case 3:
				char_val = 'M';
				break;
			case 5:
				char_val = 'R';
				break;
			case 9:
				char_val = 'W';
				break;
			case 6:
				char_val = 'S';
				break;
			case 10:
				char_val = 'Y';
				break;
			case 12:
				char_val = 'K';
				break;
			case 7:
				char_val = 'V';
				break;
			case 11:
				char_val = 'H';
				break;
			case 13:
				char_val = 'D';
				break;
			case 14:
				char_val = 'B';
				break;
			default:
				char_val = 'N';
				break;
		}
	}
	else
	{
		switch(input)
		{
			case 1:
				char_val = 'A';
				break;
			case 2:
				char_val = 'C';
				break;
			case 4:
				char_val = 'G';
				break;
			case 8:
				char_val = 'T';
				break;
			default:
				char_val = 'N';
				break;
		}

	}
	
	return char_val;
}

int ACGTvsAMB_check(int code1, int code2)
{
	if(code1==1 || code1==2 || code1==4 || code1==8)
	{
		if(code2==3 || code2==5 || code2==6 || code2==7 || code2==9 || code2==10 || code2==11 
		|| code2==12 || code2==13 || code2==14 || code2==15)
		{
			return code1;
		}
		else
		{
			return 0;
		}
	}
	else
	{

		if(code2==1 || code2==2 || code2==4 || code2==8)
		{
			if(code1==3 || code1==5 || code1==6 || code1==7 || code1==9 || code1==10 || code1==11 
			|| code1==12 || code1==13 || code1==14 || code1==15)
			{
				return code2;
			}
			else
			{
				return 0;
			}
		}
		return 0;
	}
	return 0;
}

char match_chars(char char1, char char2)
{
	int code1, code2, code_cons;
	
	if(char1==char2)
	{
		return char1;
	}
	else
	{
		if(char1=='-' || char2=='-')
		{
			if(char1=='-')
			{
				return char2;
			}

			if(char2=='-')
			{
				return char1;
			}
		}
		else
		{
			code1 = char_to_code(char1);
			code2 = char_to_code(char2);

			code_cons = code1 | code2;

			if(ACGTvsAMB_check(code1, code2)==0)
			{
				return code_to_char(code_cons);
			}
			else
			{
				return code_to_char(ACGTvsAMB_check(code1, code2));
			}

		}
	}
}

char match_chars_code(char char1, char char2)
{
	int code1, code2, code_cons;

	if(char1==char2)
	{
		return '0';
	}
	else
	{
		if(char1=='-' || char2=='-')
		{
			if(char1=='-')
			{
				return '1';
			}

			if(char2=='-')
			{
				return '2';
			}
		}
		else
		{
			return '3';

		}
	}
}

char * rev_comp_2(char * input)
 {
	int i=0,y=0;
	char temp_character;
	char * tmp_seq = (char *)malloc(sizeof(char)*strlen(input));
	
	for(i=strlen(input) ; i>-1 ; i--)
	{
	
		temp_character = input[i];
		
		if(temp_character=='A'){tmp_seq[y++]='T';}
		if(temp_character=='C'){tmp_seq[y++]='G';}
		if(temp_character=='G'){tmp_seq[y++]='C';}
		if(temp_character=='T'){tmp_seq[y++]='A';}
		if(temp_character=='N'){tmp_seq[y++]='N';}	
		if(temp_character=='U'){tmp_seq[y++]='A';}
		if(temp_character=='M'){tmp_seq[y++]='K';}
		if(temp_character=='R'){tmp_seq[y++]='Y';}
		if(temp_character=='W'){tmp_seq[y++]='W';}
		if(temp_character=='S'){tmp_seq[y++]='S';}
		if(temp_character=='Y'){tmp_seq[y++]='R';}
		if(temp_character=='V'){tmp_seq[y++]='B';}
		if(temp_character=='H'){tmp_seq[y++]='D';}
		if(temp_character=='D'){tmp_seq[y++]='H';}
		if(temp_character=='B'){tmp_seq[y++]='V';}
		if(temp_character=='-'){tmp_seq[y++]='-';}


	}

	return tmp_seq;	
 }

char * cut_edges_2(char * input, int * f_pos, int * l_pos)
 {
	int i=0,y=0;
	char * tmp = (char *)malloc(sizeof(char)*strlen(input));

	for(i=0;i<strlen(input);i++)
	{
		tmp[i]=0;
	}
	
	if(search_gaps(input,strlen(input))==1) // If TIs
	{
		*f_pos = detect_fpos (strlen(input), input);
		*l_pos = detect_lpos (*f_pos, strlen(input), input);

		if(*l_pos==0)
		{
			if(*f_pos<=strlen(input)/2)
			{
				*l_pos = strlen(input);
			}
			else//(*l_pos==0)
			{	
				*f_pos = 1 ;
				*l_pos = detect_lpos_nos (strlen(input), input);
			}
		}
	}
	else // No TIs
	{
		*f_pos = 1 ;
		*l_pos = strlen(input);
	}
	
	for(i=*f_pos-1;i<*l_pos;i++)
	{
		tmp[y++]=input[i];
	}
	tmp[y]=0;

	return tmp;
 }

consensus_struct * Consensus_with_Smith_Waterman(FILE * fp,char * array1, char * array2, int length1, int length2)
{
	int i,j;
	char * array1_ext, * array2_ext, *cut_array1, *cut_array2;
	int **T_EL;
	int value_up, value_le, value_diag;
	int max_val, max_val_i, max_val_j;
	int stop=0, cur_val;
	int * traceback_steps;
	int   traceback_i, mv;
	int array1_match_last, array1_match_first, array2_match_last, array2_match_first;
	int array1_gaps, array2_gaps;
	int cons_first, cons_last;
	int code1, code2, code_cons;
	int only_diag_start;
	char * reversed;

	int align_index=0;

	int reverse_last_position,common_last_position,alignment_length;

	char * array1_align, * array2_align;
	int array1_align_i, array2_align_i;

	char * consensus_seq;
	int consensus_seq_i;

	int length1_new;
	int length2_new;

	consensus_struct * current_consensus;

	current_consensus = (consensus_struct*)malloc(sizeof(consensus_struct));
	current_consensus -> forward_length = length2;
	current_consensus -> reverse_length = length1;
	current_consensus -> forward_sequence = (char *)malloc(sizeof(char)*length2+1);
	current_consensus -> reverse_sequence = (char *)malloc(sizeof(char)*length1+1);

	current_consensus -> forward_aligned_sequence = (char *)malloc(sizeof(char)*(length1+length2+1));
	current_consensus -> reverse_aligned_sequence = (char *)malloc(sizeof(char)*(length1+length2+1));

	current_consensus -> sequence = (char *)malloc(sizeof(char)*(length1+length2+1));

	for(i=0;i<length1+length2+1;i++)
	{
		current_consensus -> sequence[i]=0;
		current_consensus -> forward_aligned_sequence[i]=0;
		current_consensus -> reverse_aligned_sequence[i]=0;
	}

	for(i=0;i<length2;i++)
	{
		current_consensus -> forward_sequence[i]=array2[i];
	}
	current_consensus -> forward_sequence[i]=0;
	
	for(i=0;i<length1;i++)
	{
		current_consensus -> reverse_sequence[i]=array1[i];
	}
	current_consensus -> reverse_sequence[i]=0;


	array1_ext = (char *) malloc(sizeof(char)*(length1+2));
	array2_ext = (char *) malloc(sizeof(char)*(length2+2));

	array1_ext[0] = '-';
	array2_ext[0] = '-';

	cut_array1 = cut_edges_2(array1,&current_consensus->rv_cut_f,&current_consensus->rv_cut_l);

	cut_array2 = cut_edges_2(array2,&current_consensus->fw_cut_f,&current_consensus->fw_cut_l);

	
	reversed = rev_comp_2(cut_array1);
	
	for(i=0;i<strlen(cut_array1);i++)
	{
		array1_ext[i+1]=reversed[i];
		array1_ext[i+2]=0;
	}

	length1_new = strlen(cut_array1);
	
	for(i=0;i<strlen(cut_array2);i++)
	{
		array2_ext[i+1]=cut_array2[i];
   		array2_ext[i+2]=0;
	}

	length2_new = strlen(cut_array2);

	T_EL = (int **) malloc(sizeof(int *)*(length2_new+1));

	for(i=0;i<length2_new+1;i++)
	{
		T_EL[i]=(int *) malloc(sizeof(int)*(length1_new+1));

		for(j=0;j<length1_new+1;j++)
		{
			T_EL[i][j]=0;
		}
	}

	max_val = -1;
	max_val_i = -1;
	max_val_j = -1;

	for(i=1;i<length2_new+1;i++)
	{
		for(j=1;j<length1_new+1;j++)
		{

			value_up = T_EL[i-1][j] + SW_INDEL;
			value_le = T_EL[i][j-1] + SW_INDEL;

			if(array1_ext[j]==array2_ext[i])
			{
				value_diag = T_EL[i-1][j-1] + SW_MATCH;
			}
			else
			{
				value_diag = T_EL[i-1][j-1] + SW_MISMATCH;
			}

			T_EL[i][j] = max_4 (value_diag, value_up, value_le, 0);

			if(T_EL[i][j]>=max_val)
			{
				max_val = T_EL[i][j]; 
				max_val_i = i;
				max_val_j = j;
			}
	
		}
	}

	array1_match_last = max_val_j;
	array2_match_last = max_val_i;

	traceback_steps = (int *) malloc(sizeof(int)*(length1_new+length2_new + 1));
	traceback_i = length1_new + length2_new;

	array1_gaps=0;
	array2_gaps=0;
	
	only_diag_start=0;

	while(stop==0)
	{
		mv = max_3_mv (T_EL[max_val_i-1][max_val_j-1], T_EL[max_val_i-1][max_val_j], T_EL[max_val_i][max_val_j-1]);


			traceback_steps[traceback_i--] = mv;
		
			if(mv==1)
			{
				max_val_i--;
				max_val_j--;
			}
			else
			{
				if(mv==2)
				{
					max_val_i--;
					max_val_j;

					array1_gaps++;
				}
				else
				{
					max_val_i;
					max_val_j--;

					array2_gaps++;
				}
			}

			cur_val = T_EL[max_val_i][max_val_j];

			if(cur_val==0)
			{
				stop=1;
			}		
	}

	max_val_j++;
	max_val_i++;

	array1_match_first = max_val_j;
	array2_match_first = max_val_i;

	cons_first = array1_match_first - array2_match_first; 

	array1_align_i = 0;
	array2_align_i = 0;

	array1_align = (char *) malloc(sizeof(char)*(length1_new + length2_new));
	array2_align = (char *) malloc(sizeof(char)*(length1_new + length2_new));

	consensus_seq_i = 0;
	consensus_seq = (char *) malloc(sizeof(char)*(length1_new + length2_new));

	
	reverse_last_position = 0;

	for(i=0;i<array1_match_first-1;i++)
	{
		if(i<cons_first)
		{
			array1_align[array1_align_i++] = array1_ext[i+1];
			array2_align[array2_align_i++] = '-';
			
			consensus_seq[consensus_seq_i++] = array1_ext[i+1];
			current_consensus -> sequence[align_index]=array1_ext[i+1];

			current_consensus -> forward_aligned_sequence[align_index]='-';
			current_consensus -> reverse_aligned_sequence[align_index++]=array1_align[array1_align_i-1];
		
		}
		else
		{

			consensus_seq[consensus_seq_i++] = match_chars(array1_ext[i+1], array2_ext[i-cons_first+1]); 			
	
			current_consensus -> sequence[align_index]=match_chars(array1_ext[i+1], array2_ext[i-cons_first+1]);

			current_consensus -> forward_aligned_sequence[align_index]=array2_ext[i-cons_first+1];
			current_consensus -> reverse_aligned_sequence[align_index++]=array1_ext[i+1];

		}

		reverse_last_position = i;
	}

	for(i=traceback_i+1;i<length1_new + length2_new+1; i++)
	{
		if(traceback_steps[i]==1)
		{
			array1_align[array1_align_i++] = array1_ext[max_val_j++];
		    array2_align[array2_align_i++] = array2_ext[max_val_i++];


			consensus_seq[consensus_seq_i++] = match_chars(array1_align[array1_align_i-1], array2_align[array2_align_i-1]); 		
			
			current_consensus -> sequence[align_index]=match_chars(array1_align[array1_align_i-1], array2_align[array2_align_i-1]);
			
			current_consensus -> forward_aligned_sequence[align_index]=array2_align[array2_align_i-1];
			current_consensus -> reverse_aligned_sequence[align_index++]=array1_align[array1_align_i-1];


		}
		else
		{
			if(traceback_steps[i]==2)
			{
				array1_align[array1_align_i++] = '-'; //array1_ext[max_val_j++];
				array2_align[array2_align_i++] = array2_ext[max_val_i++];

				consensus_seq[consensus_seq_i++] = match_chars('-', array2_ext[max_val_i-1]);				
				
				current_consensus -> sequence[align_index]=match_chars('-', array2_ext[max_val_i-1]);
				
				current_consensus -> forward_aligned_sequence[align_index]=array2_ext[max_val_i-1];
				current_consensus -> reverse_aligned_sequence[align_index++]='-';			
			
			}
			else
			{
				array1_align[array1_align_i++] = array1_ext[max_val_j++];
				array2_align[array2_align_i++] = '-';//array2_ext[max_val_i++];
				
				consensus_seq[consensus_seq_i++] = match_chars(array1_ext[max_val_j-1],'-');			
				
				current_consensus -> sequence[align_index]=match_chars(array1_ext[max_val_j-1],'-');

				current_consensus -> forward_aligned_sequence[align_index]='-';
				current_consensus -> reverse_aligned_sequence[align_index++]=array1_ext[max_val_j-1];				
				
			}
		}

		common_last_position = i - traceback_i + reverse_last_position; 
	}
	
	j=max_val_j;
	for(i=max_val_i;i<length2_new+1;i++)
	{
		if(j<length1_new+1)
		{
			
			consensus_seq[consensus_seq_i++]=match_chars(array2_ext[i],array1_ext[j]);

			current_consensus -> sequence[align_index]=match_chars(array2_ext[i],array1_ext[j]);
			
			current_consensus -> forward_aligned_sequence[align_index]=array2_ext[i];
			current_consensus -> reverse_aligned_sequence[align_index++]=array1_ext[j];

		}
		else
		{

			consensus_seq[consensus_seq_i++]=match_chars(array2_ext[i],'-');

			current_consensus -> sequence[align_index]=match_chars(array2_ext[i],'-');

			current_consensus -> forward_aligned_sequence[align_index]=array2_ext[i];
			current_consensus -> reverse_aligned_sequence[align_index++]='-';
		
		}
		j++;
	}

	//current_consensus -> sequence[align_index]=0;
	//current_consensus -> forward_aligned_sequence[align_index]=0;
	//current_consensus -> reverse_aligned_sequence[align_index]=0;

	current_consensus -> code = (char *)malloc(sizeof(char)*align_index);

	for(i=0;i<align_index;i++)
	{
		current_consensus -> code[i] = match_chars_code(current_consensus->reverse_aligned_sequence[i],current_consensus->forward_aligned_sequence[i]);
	}
	current_consensus -> alignment_length = align_index;

	alignment_length = consensus_seq_i-1;

	current_consensus -> match_first_pos = reverse_last_position + 2;
	current_consensus -> match_last_pos = common_last_position + 1;

	return current_consensus;
}

 void process_consensus(FILE * fpC)
 {
	 int i,j,k,l,m;
	 FILE *fp;
	 FILE * fptest;
	 char inspect_file_path[200];
	 char argumentline[200];

	 char filename[200];
	 consensus_struct * current_consensus;
	
	 for(j=0;j<strlen(Consensus_Forward_Path)-7;j++)
	 {
		 filename[j]=Consensus_Forward_Path[j];
	 }

	 for(i=0;i<consensus_forward_list.size;i++)
	 {
	
		 for(k=j;k<j+strlen(consensus_forward_list.data[i]);k++)
		 {
			 filename[k]=consensus_forward_list.data[i][k-j];
			 
		 }
		 filename[k]=0;
	
		 fp=fopen(filename,"w");

		 if(DEFAULT_SCORING_SCHEME==-1)
		 {
			printf("\n   1.2: Select alignment scoring function.");
			printf("\n\n        1: DEFAULT - MATCH=3, MISMATCH=-3, GAP=-3");
			printf("\n\n        2: USER-DEFINED");
			printf("\n\n        Select function:");

			scanf("%s",argumentline);

			DEFAULT_SCORING_SCHEME = getargument(argumentline);
			
			while(DEFAULT_SCORING_SCHEME!=1 && DEFAULT_SCORING_SCHEME!=2)
			{
				printf("\n\n        %s is not valid input!",argumentline);
				printf("\n\n        Select function:");

				scanf("%s",argumentline);

				DEFAULT_SCORING_SCHEME = getargument(argumentline);
			}

			if(DEFAULT_SCORING_SCHEME==1)
			{
				SW_MATCH = SW_MATCH_DEF;
				SW_MISMATCH = SW_MISMATCH_DEF;
				SW_INDEL = SW_INDEL_DEF;
			}
			else
			{
				printf("\n           Enter MATCH score:");
				scanf("%s",argumentline);
				SW_MATCH_USR = getargumentSCORE(argumentline);

				printf("\n           Enter MISMATCH penalty:");
				scanf("%s",argumentline);
				SW_MISMATCH_USR = getargumentSCORE(argumentline);

				printf("\n           Enter GAP penalty:");
				scanf("%s",argumentline);
				SW_INDEL_USR= getargumentSCORE(argumentline);

				SW_MATCH = SW_MATCH_USR;
				SW_MISMATCH = SW_MISMATCH_USR;
				SW_INDEL = SW_INDEL_USR;
			}

		 }

		 if(CLOSEST_AMBIGUITY_REPLACEMENT==-1)
		 {
			printf("\n\n   1.3: Select mismatch handling strategy.");
			printf("\n\n        1: AMB - Every mismatch is replaced by the respective ambiguity code.");
			printf("\n\n        2: N - All mismatches are replaced by the ambiguity code N.");
			printf("\n\n        Select strategy:");

			scanf("%s",argumentline);

			CLOSEST_AMBIGUITY_REPLACEMENT = getargument(argumentline);
			
			while(CLOSEST_AMBIGUITY_REPLACEMENT!=1 && CLOSEST_AMBIGUITY_REPLACEMENT!=2)
			{
				printf("\n\n        %s is not valid input!",argumentline);
				printf("\n\n        Select strategy:");

				scanf("%s",argumentline);

				CLOSEST_AMBIGUITY_REPLACEMENT = getargument(argumentline);
			}
			
		 }

		 
		 CONSENSUS_CALC_INSPECTION=1;
		 if(CONSENSUS_CALC_INSPECTION==-1)
		 {
			printf("\n\n   1.4: Inspect consensus sequence generation.");
			printf("\n\n        1: YES\n\n        2: NO");
			printf("\n\n        Select option:");

			scanf("%s",argumentline);

			CONSENSUS_CALC_INSPECTION = getargument(argumentline);
			
			while(CONSENSUS_CALC_INSPECTION!=1 && CONSENSUS_CALC_INSPECTION!=2)
			{
				printf("\n\n        %s is not valid input!",argumentline);
				printf("\n\n        Select option:");

				scanf("%s",argumentline);

				CONSENSUS_CALC_INSPECTION = getargument(argumentline);
			}
		 }

		 if(CONSENSUS_CALC_INSPECTION==1)
		 {
			 	mkdir(consensus_inspection_folder_path,0777);
		 }
		 
		 current_consensus = Consensus_with_Smith_Waterman(fp,cons_list[i]->seq_rv_uncut, cons_list[i]->seq_fw_uncut, cons_list[i]->len_rv_uncut, cons_list[i]->len_fw_uncut);

		 fprintf(fp,"\n%s",cons_list[i]->seq_name);
		 
		 fprintf(fp,"\n%s",current_consensus ->sequence);

		 fclose(fp);

		 for(l=1;l<strlen(cons_list[i]->seq_name)+1;l++)
		 {
			 current_consensus->seq_name[l-1]=cons_list[i]->seq_name[l];
			 current_consensus->seq_name[l]=0;
		 }
	

		 /* Consensus File Format */

		 fprintf(fpC,"!>\n");
		 fprintf(fpC,"%s\n",current_consensus->seq_name);
		 fprintf(fpC,"%d\n",current_consensus->alignment_length);
		 fprintf(fpC,"%s\n",current_consensus->sequence);		 
		 for(l=0;l<current_consensus->alignment_length;l++)
		 {
			 fprintf(fpC,"%c",current_consensus->code[l]);
		 }
		 fprintf(fpC,"\n");
		 fprintf(fpC,"%d\n",current_consensus ->forward_length);
		 fprintf(fpC,"%d\n",current_consensus ->fw_cut_f);
		 fprintf(fpC,"%d\n",current_consensus ->fw_cut_l);
		 fprintf(fpC,"%d\n",current_consensus ->reverse_length);
		 fprintf(fpC,"%d\n",current_consensus ->rv_cut_f);
		 fprintf(fpC,"%d\n",current_consensus ->rv_cut_l);
		 fprintf(fpC,"%d\n",current_consensus ->match_first_pos);
		 fprintf(fpC,"%d\n",current_consensus ->match_last_pos);

		 /*************************/

		 if(CONSENSUS_CALC_INSPECTION==1)
		 {
			 for(l=0;l<strlen(consensus_inspection_folder_path);l++)
			 {
				 inspect_file_path[l]=consensus_inspection_folder_path[l];

			 }
			 inspect_file_path[l++]='/';
			 for(k=l;k<strlen(consensus_inspection_folder_path)+strlen(cons_list[i]->seq_name);k++)
			 {
				 inspect_file_path[k]=cons_list[i]->seq_name[k-strlen(consensus_inspection_folder_path)];
			 }
			 inspect_file_path[k++]='.';inspect_file_path[k++]='f';inspect_file_path[k++]='a';
			 inspect_file_path[k++]='s';inspect_file_path[k++]=0;

			 fptest = fopen(inspect_file_path,"w");

			 fprintf(fptest,"\n>reversed\n");
			 fprintf(fptest,"%s",current_consensus ->reverse_aligned_sequence);

			 fprintf(fptest,"\n>forward\n");
			 fprintf(fptest,"%s",current_consensus ->forward_aligned_sequence);

			 fprintf(fptest,"\n%s(consensus)",cons_list[i]->seq_name);		 
			 fprintf(fptest,"\n%s",current_consensus ->sequence);
			 // TODO REMOVE
			 //fprintf(fptest,"\n%s(codes)\n",cons_list[i]->seq_name);
			 //for(m=0;m<current_consensus->alignment_length;m++)
			 //{
			 //fprintf(fptest,"%c",current_consensus ->code[m]);
			 //}
			 fclose(fptest);
		 }		 
	 }
 }

 void read_consensus_out()
{
	FILE * fp;
	int cons_num=-1;
	char tmp;
	
	int i,l;
	char trash[200];

	fp = fopen(consensus_out_path,"r");

	if(fp!=NULL)
	{
		tmp = fgetc(fp);
		cons_num = 0 ;
		while(tmp!=EOF)
		{
			if(tmp=='!')
			{
				cons_num ++;
			}
			tmp = fgetc(fp);
		}

		fclose(fp);

		consensus_out_size = cons_num;
		consensus_out = (consensus_struct**)malloc(sizeof(consensus_struct*)*cons_num);

		fp = fopen(consensus_out_path,"r");

		
		for(i=0;i<cons_num;i++)
		{
			consensus_out[i] = (consensus_struct*)malloc(sizeof(consensus_struct));
		
			fscanf(fp,"%s",trash);

			fscanf(fp,"%s",trash);
			for(l=0;l<strlen(trash);l++)
			{
				consensus_out[i]->seq_name[l]=trash[l];
			}
			consensus_out[i]->seq_name[l]=0;

			fscanf(fp,"%d",&consensus_out[i]->alignment_length);
			
			consensus_out[i]->sequence = (char *)malloc(sizeof(char)*(consensus_out[i]->alignment_length+1));
			fscanf(fp,"%s",consensus_out[i]->sequence);
			consensus_out[i]->sequence[consensus_out[i]->alignment_length]=0;

			consensus_out[i]->code = (char *)malloc(sizeof(char)*consensus_out[i]->alignment_length);
			trash[0]=fgetc(fp);

			for(l=0;l<consensus_out[i]->alignment_length;l++)
			{
				consensus_out[i]->code[l] = fgetc(fp);
			}
			fscanf(fp,"%d",&consensus_out[i] ->forward_length);
			fscanf(fp,"%d",&consensus_out[i] ->fw_cut_f);
			fscanf(fp,"%d",&consensus_out[i] ->fw_cut_l);
			fscanf(fp,"%d",&consensus_out[i] ->reverse_length);
			fscanf(fp,"%d",&consensus_out[i] ->rv_cut_f);
			fscanf(fp,"%d",&consensus_out[i] ->rv_cut_l);
			fscanf(fp,"%d",&consensus_out[i] ->match_first_pos);
			fscanf(fp,"%d",&consensus_out[i] ->match_last_pos);
		}		
		
		fclose(fp);
	}
}
 

 int main(int argc, char * argv[])
 {

 	int max_F_num = 0 , i,j, FASTA_seq_len=0, FASTA_new_seq_len=0, mode=-1;
 	char done;
	int size_ext;
	char temp[100],temp2[100];
	char ambigcode;

 	FILE *fp=NULL;
 	FILE *fpN=NULL;
	FILE *fpA=NULL;
	FILE *fpC=NULL;
 	
	print_label();
 	
 	mode=print_menu(); 	
    
	
	if (mode==CONSENSUS_RUN)
    {	
		fpC = fopen(consensus_out_file,"w");

		read_data_consensus();

		process_consensus(fpC);

		fclose(fpC);

		printf("\n\n\n\n   => Consensus Sequence Generation Complete! \n\n");
		printf("\n      Total number of generated consensus files: %d ! \n\n",consensus_forward_list.size);

		//if(CONSENSUS_CALC_INSPECTION==1)
		//{
		//	printf("\n      The ../CONSENSUS/INSPECT folder is up to date! \n\n");
		//}
		//else
		//{
		//	printf("\n      The ../CONSENSUS/INSPECT folder (if exists) is not up to date! \n\n");
		//}

			
		printf("\n  Press Enter to exit...\n\n");
		scanf("%c",&done); scanf("%c",&done);
		
		return 0;    
    
    }

    if (mode==PRE_ALIGNMENT_RUN)
    {
		read_data ();

		max_F_num = total_number_of_files;
		
		fp=fopen(pre_alignment_fas_path,"w");

		create_pre_multalign_file(max_F_num,fp);

		fclose(fp);	
				
		fp=fopen(lut_out_path,"w");

		print_lut_out(max_F_num,fp);

		fclose(fp);
	
		fp=fopen(pre_alignment_out_path,"w");

		store_pre_multalign_data(max_F_num,fp);

		fclose(fp);
		
		printf("\n\n\n\n   => Preliminary File Generation Complete! \n\n");

		if(max_F_num==0)
		{
			printf("\n      WARNING: The Preliminary File does not contain any sequences.\n\n");
		}
		else
		{
			if(max_F_num==1)
			{
				printf("\n      The Preliminary File contains only %d sequence.\n",max_F_num);

				if(mode1_load_F==1)
				{
					printf("\n      The sequence was retrieved from the FORWARD folder.\n\n");
				}
				if(mode1_load_R==1)
				{
					printf("\n      The sequence was retrieved from the REVERSE folder.\n\n");
				}
				if(mode1_load_C==1)
				{
					printf("\n      The sequence was retrieved from the CONSENSUS folder.\n\n");
				}				
			}
			else
			{
				printf("\n      The Preliminary File contains a total of %d sequences.\n\n",max_F_num);

				printf("\n      The sequences were retrieved as follows:\n");

				if(mode1_load_F!=0)
				{
					if(mode1_load_F==1)
					{
						printf("\n      %d sequence was retrieved from the FORWARD folder.\n",mode1_load_F);
					}
					else
					{
						printf("\n      %d sequences were retrieved from the FORWARD folder.\n",mode1_load_F);
					}
				}
				if(mode1_load_R!=0)
				{					
					if(mode1_load_R==1)
					{
						printf("\n      %d sequence was retrieved from the REVERSE folder.\n",mode1_load_R);
					}
					else
					{
						printf("\n      %d sequences were retrieved from the REVERSE folder.\n",mode1_load_R);
					}
				}
				if(mode1_load_C!=0)
				{				
					if(mode1_load_C==1)
					{
						printf("\n      %d sequence was retrieved from the CONSENSUS folder.\n",mode1_load_C);
					}
					else
					{
						printf("\n      %d sequences were retrieved from the CONSENSUS folder.\n",mode1_load_C);
					}
				}
				printf("\n");
			}
			  
		}		 

		printf("\n  Press Enter to exit...\n\n");
		scanf("%c",&done);
		
		return 0;    
    
    }
    
    if (mode==AMBIGUITIES_RUN || mode==DIVERSITY_RUN)
    {		

     	fp = fopen (lut_out_path,"r");
	 	if(fp==NULL) 
		{ 
			printf("\n\n\n\n   => ../%s file is missing.\n\n",lut_out_path);

			printf("\n  Press Enter to exit...\n\n");
			scanf("%c",&done);scanf("%c",&done);

			exit(0); 
		}
	 	
	 	fscanf(fp,"%d",&max_F_num);	
	 	
	 	seq_d_table = (seq_f*)malloc(sizeof(seq_f)*max_F_num);
	 	
	 	seq_n_table = (char**)malloc(sizeof(char *)*max_F_num);

		for (i=0;i<max_F_num;i++)
		{
			seq_n_table[i]=(char*)malloc(sizeof(char)*50);
		}
	
		read_lut(max_F_num,fp);
	 		
	 	fclose(fp);
	 	
	 	fp = fopen (pre_alignment_out_path,"r");
	 	if(fp==NULL)
		{ 
			printf("\n\n\n\n   => ../%s file is missing.\n\n",pre_alignment_out_path);				

			printf("\n  Press Enter to exit...\n\n");
			scanf("%c",&done);scanf("%c",&done);

			exit(0); 
		}
	 
	 	read_pre_alignment(max_F_num,fp);
	 	
	 	fclose(fp);

		fp = fopen (ma1_name,"r");	 	
		if(fp==NULL)
		{ 
			printf("\n\n\n\n   => ../%s file is missing.\n\n",ma1_name);				

			printf("\n  Press Enter to exit...\n\n");
			scanf("%c",&done);scanf("%c",&done);

			exit(0); 
		}

	 
	 	FASTA_seq_len=get_FASTA_seq_length (fp);
	 	
	 	fclose(fp);
	 	
	 	fp = fopen (ma1_name,"r");
	 	
	 	size_ext=read_multiple_alignment(max_F_num,FASTA_seq_len,fp,1);
	 	
	  	fclose(fp);

		read_consensus_out(); 

	 }

 	
 	if (run_arg==AMBIGUITIES_RUN)
 	{
		ambtotal = 0;
		ambrtotal = 0;

		ambigcode='N';
		calc_per_col_div_ambig(ambigcode, max_F_num,FASTA_seq_len,size_ext);

		ambigcode='K';
		calc_per_col_div_ambig(ambigcode, max_F_num,FASTA_seq_len,size_ext);

		ambigcode='M';
		calc_per_col_div_ambig(ambigcode, max_F_num,FASTA_seq_len,size_ext);

		ambigcode='R';
		calc_per_col_div_ambig(ambigcode, max_F_num,FASTA_seq_len,size_ext);

		ambigcode='W';
		calc_per_col_div_ambig(ambigcode, max_F_num,FASTA_seq_len,size_ext);
		 
		ambigcode='S';
		calc_per_col_div_ambig(ambigcode, max_F_num,FASTA_seq_len,size_ext);

		ambigcode='Y';
		calc_per_col_div_ambig(ambigcode, max_F_num,FASTA_seq_len,size_ext);
		
		ambigcode='V';
		calc_per_col_div_ambig(ambigcode, max_F_num,FASTA_seq_len,size_ext);
		
		ambigcode='H';
		calc_per_col_div_ambig(ambigcode, max_F_num,FASTA_seq_len,size_ext);
		
		ambigcode='D';
		calc_per_col_div_ambig(ambigcode, max_F_num,FASTA_seq_len,size_ext);
	
		ambigcode='B';
		calc_per_col_div_ambig(ambigcode, max_F_num,FASTA_seq_len,size_ext);
 		

		printf("\n\n\n\n   => Ambiguity Character Detection Complete!\n\n\n");

		printf("      The alignment \"%s\" comprises:\n\n", ma1_name);		
		printf("      %d sequences,\n\n", size_ext);		
		printf("      %d sites, and \n\n",FASTA_seq_len);
		printf("      %d ambiguities.\n\n\n",ambtotal);

		if(ambrtotal==0)
		{
			printf("      No ambiguity reports have been generated!\n\n\n");
		}
		else
		{
			if(ambrtotal==1)
			{
				printf("      %d ACD report has been generated!\n\n\n",ambrtotal);
			}
			else
			{
				printf("      %d ACD reports have been generated!\n\n\n",ambrtotal);
			}		

		}
	
		printf("\n  Press Enter to exit...\n\n");
		scanf("%c",&done);scanf("%c",&done);

 		return 0;
 	}
 	else
 	{	
		if(run_arg==DIVERSITY_RUN)
 		{
			
			fp = fopen (ma2_name,"r"); 		
			if(fp==NULL)
			{ 
				printf("\n\n\n\n   => ../%s file is missing.\n\n",ma2_name);				

				printf("\n  Press Enter to exit...\n\n");
				scanf("%c",&done);scanf("%c",&done);

				exit(0); 
			}			
 
 			FASTA_new_seq_len=get_FASTA_seq_length (fp);
 			ma2_length = FASTA_new_seq_len;

 			fclose(fp);
 			
 			fp = fopen (ma2_name,"r");
 	
 			size_ext=read_multiple_alignment(max_F_num,FASTA_new_seq_len,fp,2);
 	
 			fclose(fp);

			adjust_threshold(size_ext);

			ambigcode='N';
			alter_N_report_path(ambigcode);
			fpN= fopen (N_Report_path,"r");
			if(fpN!=NULL)
			{
				read_X_report(ambigcode,max_F_num,fpN); 		
 				fclose(fpN);
			} 	

			ambigcode='K';
			alter_N_report_path(ambigcode);
			fpN= fopen (N_Report_path,"r");
			if(fpN!=NULL)
			{
				read_X_report(ambigcode,max_F_num,fpN); 		
 				fclose(fpN);
			} 			
			
			ambigcode='M';
			alter_N_report_path(ambigcode);
			fpN= fopen (N_Report_path,"r");
			if(fpN!=NULL)
			{
				read_X_report(ambigcode,max_F_num,fpN); 		
 				fclose(fpN);
			}

			ambigcode='R';
			alter_N_report_path(ambigcode);
			fpN= fopen (N_Report_path,"r");
			if(fpN!=NULL)
			{
				read_X_report(ambigcode,max_F_num,fpN); 		
 				fclose(fpN);
			}

			ambigcode='W';
			alter_N_report_path(ambigcode);
			fpN= fopen (N_Report_path,"r");
			if(fpN!=NULL)
			{
				read_X_report(ambigcode,max_F_num,fpN); 		
 				fclose(fpN);
			}		 

			ambigcode='S';
			alter_N_report_path(ambigcode);
			fpN= fopen (N_Report_path,"r");
			if(fpN!=NULL)
			{
				read_X_report(ambigcode,max_F_num,fpN); 		
 				fclose(fpN);
			}
			
			ambigcode='Y';
			alter_N_report_path(ambigcode);
			fpN= fopen (N_Report_path,"r");
			if(fpN!=NULL)
			{
				read_X_report(ambigcode,max_F_num,fpN); 		
 				fclose(fpN);
			}		
			
			ambigcode='V';
			alter_N_report_path(ambigcode);
			fpN= fopen (N_Report_path,"r");
			if(fpN!=NULL)
			{
				read_X_report(ambigcode,max_F_num,fpN); 		
 				fclose(fpN);
			}		
			
			ambigcode='H';
			alter_N_report_path(ambigcode);
			fpN= fopen (N_Report_path,"r");
			if(fpN!=NULL)
			{
				read_X_report(ambigcode,max_F_num,fpN); 		
 				fclose(fpN);
			}		
			
			ambigcode='D';
			alter_N_report_path(ambigcode);
			fpN= fopen (N_Report_path,"r");
			if(fpN!=NULL)
			{
				read_X_report(ambigcode,max_F_num,fpN); 		
 				fclose(fpN);
			}	
			
			ambigcode='B';
			alter_N_report_path(ambigcode);
			fpN= fopen (N_Report_path,"r");
			if(fpN!=NULL)
			{
				read_X_report(ambigcode,max_F_num,fpN); 		
 				fclose(fpN);
			} 		
			
			
 			
 			fp= fopen (D_Report_path,"w");
			fpA = fopen (A_Report_path,"w");

			report_label(2,'x',fp,workspace_name_g,ma2_filename);
			report_label(3,'x',fpA,workspace_name_g,ma2_filename);

 			fprintf(fp, "\n\t	ALIGNMENT SITE \t\t SITE REPORT \t\t SEQUENCE NAME \t\t\t\t      CHROMATOGRAM POSITION \n   __________________________________________________________________________________________________________________________________ \n");
 			fprintf(fpA, "\n\t	ALIGNMENT SITE \t\t AMBIGUITY CHARACTER \t\t SEQUENCE NAME \n   _____________________________________________________________________________________ \n");
 			
			calc_per_col_div_MA2(max_F_num,FASTA_new_seq_len,fp,fpA,FASTA_seq_len,size_ext);

 			fclose(fp);			
 			fclose(fpA);

			printf("\n\n\n\n   => Polymorphic Site Detection Complete!\n\n\n");


			printf("      The alignment \"%s\" comprises:\n\n", ma2_name);		
			printf("      %d sequences,\n\n", size_ext);		
			printf("      %d sites, and\n\n", FASTA_new_seq_len);
			printf("      %d ambiguities.\n\n",total_amb_g);
			
			printf("      %d of the sites are polumorphic.\n\n\n",total_unalcol_g);
			
			printf("      A PSD and a post-PSD ACD report have been generated! \n\n");

			printf("\n  Press Enter to exit...\n\n");
			scanf("%c",&done);scanf("%c",&done);

  			return 0;
 		}
 	} 
 	return 0;
 }
